@@ -14,9 +14,11 @@
 
 Annotate::Annotate(Git* parent, QObject* guiObj) : QObject(parent) {
 
+	EM_INIT(exAnnCanceled, "Canceling annotation");
+
 	git = parent;
 	gui = guiObj;
-	cancelingAnnotate = annotateRunning = false;
+	cancelingAnnotate = annotateRunning = annotateActivity = false;
 	valid = canceled = false;
 
 	patchProcBuf.reserve(1000000); // avoid repeated reallocation, will be big!
@@ -39,21 +41,20 @@ const FileAnnotation* Annotate::lookupAnnotation(SCRef sha, SCRef fn) {
 	// ok, we are not lucky. Check for an ancestor before to give up
 	int shaIdx;
 	const QString ancestorSha = getAncestor(sha, fileName, &shaIdx);
-	if (ancestorSha.isEmpty())
-		return NULL;
-
-	it = ah.find(ancestorSha);
-	if (it != ah.constEnd())
-		return &(it.data());
-
+	if (!ancestorSha.isEmpty()) {
+		it = ah.find(ancestorSha);
+		if (it != ah.constEnd())
+			return &(it.data());
+	}
 	return NULL;
 }
 
 bool Annotate::startPatchProc(SCRef buf, SCRef fileName) {
 
-	const QString cmd("git diff-tree -r -m --patch-with-raw "
-	                  "--no-commit-id --stdin -- " + fileName);
-	patchProc.setArguments(QStringList::split(' ', cmd));
+	QString cmd("git diff-tree -r -m --patch-with-raw --no-commit-id --stdin --");
+	QStringList args(QStringList::split(' ', cmd));
+	args.append(fileName); // handle file name with spaces case
+	patchProc.setArguments(args);
 	patchProc.setWorkingDirectory(git->workDir);
 	patchProcBuf = "";
 	return patchProc.launch(buf);
@@ -68,7 +69,10 @@ void Annotate::on_patchProc_readFromStdout() {
 	patchProcBuf.append(tmp);
 }
 
-bool Annotate::stop() {
+void Annotate::deleteWhenDone() {
+
+	if (!EM_IS_PENDING(exAnnCanceled))
+		EM_RAISE(exAnnCanceled);
 
 	if (annotateRunning)
 		cancelingAnnotate = true;
@@ -76,7 +80,15 @@ bool Annotate::stop() {
 	if (patchProc.isRunning())
 		patchProc.tryTerminate();
 
-	return !annotateRunning;
+	on_deleteWhenDone();
+}
+
+void Annotate::on_deleteWhenDone() {
+
+	if (!(annotateRunning || EM_IS_PENDING(exAnnCanceled)))
+		deleteLater();
+	else
+		QTimer::singleShot(20, this, SLOT(on_deleteWhenDone()));
 }
 
 void Annotate::on_progressTimer_timeout() {
@@ -156,12 +168,15 @@ void Annotate::slotComputeDiffs() {
 	valid = !(isError || cancelingAnnotate);
 	canceled = cancelingAnnotate;
 	cancelingAnnotate = annotateRunning = false;
-	git->annotateExited(this);
+	if (canceled)
+		deleteWhenDone();
+	else
+		git->annotateExited(this);
 
-// 	StrVect::const_iterator it(histRevOrder.constBegin());
-// 	do {
-// 		dbg(*it); dbg(ah[*it].fileSha);
-// 	} while (++it != histRevOrder.constEnd());
+//	StrVect::const_iterator it(histRevOrder.constBegin());
+//	do {
+//		dbg(*it); dbg(ah[*it].fileSha);
+//	} while (++it != histRevOrder.constEnd());
 }
 
 void Annotate::annotateFileHistory(SCRef fileName, bool buildPatchScript) {
@@ -172,14 +187,14 @@ void Annotate::annotateFileHistory(SCRef fileName, bool buildPatchScript) {
 	do {
 		--it;
 		doAnnotate(fileName, *it, buildPatchScript);
-	} while (it != histRevOrder.constBegin() && !isError);
+	} while (it != histRevOrder.constBegin() && !isError && !cancelingAnnotate);
 }
 
 void Annotate::doAnnotate(SCRef fileName, SCRef sha, bool buildPatchScript) {
 // all the parents annotations must be valid here
 
 	FileAnnotation* fa = getFileAnnotation(sha);
-	if (fa == NULL || fa->isValid || isError)
+	if (fa == NULL || fa->isValid || isError || cancelingAnnotate)
 		return;
 
 	const Rev* r = git->revLookup(sha, fh); // historyRevs
@@ -256,14 +271,21 @@ FileAnnotation* Annotate::getFileAnnotation(SCRef sha) {
 void Annotate::setInitialAnnotation(SCRef fileName, SCRef sha, FileAnnotation* fa) {
 
 	QString f;
-	SCRef fileSha(git->getFileSha(fileName, sha));
+	SCRef fileSha(git->getFileSha(fileName, sha)); // calls Qt event loop
+	if (cancelingAnnotate)
+		return;
+
 	if (fileSha.isEmpty()) {
 		dbp("ASSERT in setInitialAnnotation: empty file of initial rev %1", sha);
 		isError = true;
 		return;
 	}
 	ah[sha].fileSha = fileSha;
+
 	git->getFile(fileName, sha, NULL, &f); // blocking call, calls Qt event loop
+	if (cancelingAnnotate)
+		return;
+
 	int lineNum = f.contains('\n');
 	if (!f.endsWith("\n")) // No newline at end of file
 		lineNum++;
@@ -329,7 +351,7 @@ void Annotate::setAnnotation(SCRef diff, SCRef author, SCList prevAnn,
 			// instead QValueList::at() starts from 0
 			if (num <= 0) { // file is deleted
 				if (num < 0)
-					qDebug("ASSERT processDiff: start line number is %i", num);
+					dbp("ASSERT processDiff: start line number is %1", num);
 				newAnn.clear();
 				return;
 			}
@@ -366,6 +388,8 @@ void Annotate::setAnnotation(SCRef diff, SCRef author, SCList prevAnn,
 			// message is locale dependent, so just test the space after '\'
 			if (line[1] == ' ')
 				break;
+			else
+				; // fall through
 		default:
 			++cur;
 			break;
@@ -387,8 +411,13 @@ const QString Annotate::getNextPatch(QString& patchFile, SCRef fileName, SCRef s
 	if (sha == QGit::ZERO_SHA) {
 		// diff-tree --stdin doesn't work with working dir so get it
 		// directly. We don't need file sha info because is ZERO_SHA
-		QString runCmd("git diff-index -r -m -p HEAD -- " + fileName), runOutput;
+		QString runOutput;
+		QString runCmd("git diff-index -r -m -p HEAD -- " + QGit::QUOTE_CHAR +
+				fileName + QGit::QUOTE_CHAR);
+
 		git->run(runCmd, &runOutput);
+		if (cancelingAnnotate)
+			return "";
 
 		// restore removed head line of next patch
 		const QString nextHeader("\n:100644 100644 " + QGit::ZERO_SHA + ' '
@@ -454,75 +483,77 @@ bool Annotate::getRange(SCRef sha, RangeInfo* r) {
 	return true;
 }
 
-void Annotate::updateCrossRanges(SCRef chunk, bool reverse,
-			int fileLenght, int ofs, RangeInfo* r) {
+void Annotate::updateCrossRanges(SCRef chunk, bool rev, int fileLen, int ofs, RangeInfo* r) {
 
 /* here the deal is to fake a file that will be modified by chunk, the
    file must contain also the whole output range.
 
    Then we apply an annotation step and see what happens...
 
-   In no reverse case we check what happens to a selected range after
-   patching, so we mark only the lines of selected range.
+   First we mark each line of the file with the corresponding line number,
+   then apply the patch and check where the lines have been moved around.
+
+   Now we have to cases:
+
+   - In reverse case we infer the before-patch range knowing the after-patch range.
+     So we check what lines we have in the region corresponding to after-patch range.
+
+   - In forward case we infer the after-patch range knowing the before-patch range.
+     So we scan the resulting annotation to find line numbers corresponding to the
+     before-patch range.
 
-   In reverse case we check where the selected range belongs, i.e. the origin
-   range that, with the patch applied will be mapped in the selected range.
-   In this case we need to mark each line of the file, then after patching
-   we check the selected range lines's mark to find the origin range.
 */
-
 	// because of the padding the file first line number will be
-	// fileFirstLineNr = oldLineId - beforePadding = fileOffset + 1
-	QStringList ann;
-	for (int lineNum = ofs + 1; lineNum <= ofs + fileLenght; lineNum++) {
+	//
+	// fileFirstLineNr = newLineId - beforePadding = fileOffset + 1
+	QStringList beforeAnn, afterAnn;
+	for (int lineNum = ofs + 1; lineNum <= ofs + fileLen; lineNum++)
+		beforeAnn.append(QString::number(lineNum));
 
-		SCRef strNum(QString::number(lineNum));
-		if (reverse)
-			ann.append(strNum);
-		else
-			ann.append((lineNum  < r->start || lineNum > r->end) ? "" : strNum);
-	}
 	const QString fakedAuthor("*");
-	QStringList tmpAnn;
-	setAnnotation(chunk, fakedAuthor, ann, tmpAnn, ofs);
-	r->modified = false;
-	int newStart = 0, newEnd = 0;
-	if (reverse) {
+	setAnnotation(chunk, fakedAuthor, beforeAnn, afterAnn, ofs);
+	int newStart = ofs + 1;
+	int newEnd = ofs + fileLen;
 
-		// the first file line number is ofs + 1, so the line start, will be
-		// at start - (ofs + 1) because at() starts from 0
-		QStringList::const_iterator it1(tmpAnn.at(r->start - ofs - 1));
-		QStringList::const_iterator it2(tmpAnn.at(r->end - ofs - 1));
+	if (rev) {
+		// let's see what line number we have at given range interval limits.
+		// at() counts from 0.
+		QStringList::const_iterator itStart(afterAnn.at(r->start - ofs - 1));
+		QStringList::const_iterator itEnd(afterAnn.at(r->end - ofs - 1));
 
-		bool firstIsPatch = (*it1 == fakedAuthor);
-		bool lastIsPatch = (*it2 == fakedAuthor);
+		bool leftExtended = (*itStart == fakedAuthor);
+		bool rightExtended = (*itEnd == fakedAuthor);
 
-		while (*it1 == fakedAuthor) {
-			if (it1 == tmpAnn.constBegin()) {
-				it1 == tmpAnn.constEnd();
+		// if range boundary is a line added by the patch
+		// we consider inclusive and extend the range
+		++itStart;
+		do {
+			--itStart;
+			if (*itStart != fakedAuthor) {
+				newStart = (*itStart).toInt();
 				break;
 			}
-			--it1;
-		}
-		while (*it2 == fakedAuthor) {
-			++it2;
-			if (it2 == tmpAnn.constEnd())
-				break;
-		}
-		newStart = (it1 != tmpAnn.constEnd()) ? (*it1).toInt() : ofs + 1;
-		newEnd = (it2 != tmpAnn.constEnd()) ? (*it2).toInt() : ofs + fileLenght;
+		} while (itStart != afterAnn.constBegin());
 
-		if (firstIsPatch)
+		while (itEnd != afterAnn.constEnd()) {
+
+			if (*itEnd != fakedAuthor) {
+				newEnd = (*itEnd).toInt();
+				break;
+			}
+			++itEnd;
+		}
+		if (leftExtended && *itStart != fakedAuthor)
 			newStart++;
 
-		if (lastIsPatch)
+		if (rightExtended && itEnd != afterAnn.constEnd())
 			newEnd--;
 
-		r->modified = (firstIsPatch || lastIsPatch);
+		r->modified = (leftExtended || rightExtended);
 
 		if (!r->modified) { // check for consecutive sequence
-			for (int i = r->start; i <= r->end; ++i, ++it1)
-				if (i - r->start != (*it1).toInt() - newStart) {
+			for (int i = r->start; i <= r->end; ++i, ++itStart)
+				if (i - r->start != (*itStart).toInt() - newStart) {
 					r->modified = true;
 					break;
 				}
@@ -530,58 +561,45 @@ void Annotate::updateCrossRanges(SCRef chunk, bool reverse,
 		if (newStart > newEnd) // selected range is whole inside new added lines
 			newStart = newEnd = 0;
 
-	} else { // forward apply case
+	} else { // forward case
 
-		// first thing is to find delimiting numbers if any
-		QStringList::const_iterator it(tmpAnn.constBegin());
-		QStringList::const_iterator it1(tmpAnn.constEnd()), it2;
+		// scan afterAnn to check for before-patch range boundaries
+		QStringList::const_iterator itStart(afterAnn.constEnd());
+		QStringList::const_iterator itEnd(afterAnn.constEnd());
 
-		for (int lineNum = ofs + 1; it != tmpAnn.constEnd(); ++lineNum, ++it) {
+		QStringList::const_iterator it(afterAnn.constBegin());
+		for (int lineNum = ofs + 1; it != afterAnn.constEnd(); ++lineNum, ++it) {
 
-			if (!(*it).isEmpty() && *it != fakedAuthor) {
-				if (it1 == tmpAnn.constEnd()) { // still init value
-					it1 = it;
+			if (*it != fakedAuthor) {
+
+				if ((*it).toInt() <= r->start) {
 					newStart = lineNum;
+					itStart = it;
+				}
+				if (  (*it).toInt() >= r->end
+				    && itEnd == afterAnn.constEnd()) { // one-shot
+					newEnd = lineNum;
+					itEnd = it;
 				}
-				it2 = it;
-				newEnd = lineNum;
 			}
 		}
-		if (it1 == tmpAnn.constEnd()) { // selected range has been deleted!
-			r->start = r->end = 0;
-			r->modified = true;
-			return;
-		}
-		if ((*it1).toInt() != r->start) { // start range delimiter has been deleted
+		if (itStart != afterAnn.constEnd() && (*itStart).toInt() < r->start)
+			newStart++;
 
-			// we use a greedy policy and take the biggest range among
-			// possible ranges. If range boundary is a line added by the patch
-			// we consider them inclusive.
-			--it1;
-			while (*it1 == fakedAuthor) {
-				newStart--;
-				if (it1 == tmpAnn.constBegin())
-					break;
-				--it1;
-			}
-			r->modified = true;
-		}
-		if ((*it2).toInt() != r->end) { // end range delimiter has been deleted
-			++it2;
-			while (*it2 == fakedAuthor) {
-				newEnd++;
-				++it2;
-				if (it2 == tmpAnn.constEnd())
-					break;
-			}
-			r->modified = true;
-		}
-		if (!r->modified) // in this case it1 and it2 have not been modified
-			for (int i = r->start; it1 != it2; ++it1, i++)
-				if ((*it1).toInt() != i) { // check for consecutive sequence
-					r->modified = true;
-					break;
+		if (itEnd != afterAnn.constEnd() && (*itEnd).toInt() > r->end)
+			newEnd--;
+
+		r->modified = (itStart == afterAnn.constEnd() || itEnd == afterAnn.constEnd());
+
+		if (!r->modified) { // check for consecutive sequence
+			for (int i = r->start; i <= r->end; ++itStart, i++)
+				if ((*itStart).toInt() != i) {
+				r->modified = true;
+				break;
 				}
+		}
+		if (newStart > newEnd) // selected range has been deleted
+			newStart = newEnd = 0;
 	}
 	r->start = newStart;
 	r->end = newEnd;
@@ -642,8 +660,8 @@ void Annotate::updateRange(RangeInfo* r, SCRef diff, bool reverse) {
 
 		// case 2: patch range before our range
 		if (patchEnd < r->start) {
- 			r->start += ((reverse) ? -lineNumDiff : lineNumDiff);
- 			r->end += ((reverse) ? -lineNumDiff : lineNumDiff);
+			r->start += ((reverse) ? -lineNumDiff : lineNumDiff);
+			r->end += ((reverse) ? -lineNumDiff : lineNumDiff);
 			continue;
 		}
 		// case 3: the patch is whole inside our range
@@ -683,10 +701,33 @@ void Annotate::updateRange(RangeInfo* r, SCRef diff, bool reverse) {
 
 const QString Annotate::getAncestor(SCRef sha, SCRef fileName, int* shaIdx) {
 
-	SCRef fileSha(git->getFileSha(fileName, sha));
-	if (fileSha.isEmpty()) {
-		dbp("ASSERT in computeRanges: empty file from %1", sha);
-		return "";
+	QString fileSha;
+
+	try {
+		annotateActivity = true;
+		EM_REGISTER(exAnnCanceled);
+
+		fileSha = git->getFileSha(fileName, sha); // calls qApp->processEvents()
+		if (fileSha.isEmpty()) {
+			dbp("ASSERT in getAncestor: empty file from %1", sha);
+			return "";
+		}
+		EM_REMOVE(exAnnCanceled);
+		annotateActivity = false;
+
+	} catch(int i) {
+
+		EM_REMOVE(exAnnCanceled);
+		annotateActivity = false;
+
+		if (EM_MATCH(i, exAnnCanceled, "getting ancestor")) {
+			EM_THROW_PENDING;
+			return "";
+		}
+		const QString info("Exception \'" + EM_DESC(i) + "\' "
+		                   "not handled in Annotation lookup...re-throw");
+		dbs(info);
+		throw;
 	}
 	// NOTE: more then one revision could have the same file sha as our
 	// input revision. This happens if the patch is reverted or if the patch
@@ -699,12 +740,21 @@ const QString Annotate::getAncestor(SCRef sha, SCRef fileName, int* shaIdx) {
 		if (fa.fileSha == fileSha)
 			return histRevOrder[*shaIdx];
 	}
-	dbp("ASSERT in computeRanges: ancestor of %1 not found", sha);
+	dbp("ASSERT in getAncestor: ancestor of %1 not found", sha);
 	return "";
 }
 
-const QString Annotate::computeRanges(SCRef sha, int rangeStart,
-		int rangeEnd, SCRef target) { // blocking call, no qApp->processEvents() call
+/*
+  Range filtering uses only annotated added/removed lines to compute ranges,
+  patch content is never checked, this algorithm is fast but fails in case of
+  code shuffle; if a patch moves some code between two independents part of the
+  same file this will be interpreted as a delete of origin code. New added code
+  is not recognized as the same of old one because content is not checked.
+
+  Only checking for copies would fix the corner case, but implementation
+  is too difficult, so better accept this design limitation for now.
+*/
+const QString Annotate::computeRanges(SCRef sha, int rangeStart, int rangeEnd, SCRef target) {
 
 	rangeMap.clear();
 
@@ -806,7 +856,7 @@ const QString Annotate::computeRanges(SCRef sha, int rangeStart,
 	return ancestor;
 }
 
-bool Annotate::seekPosition(int& paraFrom, int& paraTo, SCRef fromSha, SCRef toSha) {
+bool Annotate::seekPosition(int* paraFrom, int* paraTo, SCRef fromSha, SCRef toSha) {
 
 	if (paraFrom == 0 || fromSha == toSha)
 		return true;
@@ -814,15 +864,15 @@ bool Annotate::seekPosition(int& paraFrom, int& paraTo, SCRef fromSha, SCRef toS
 	QMap<QString, RangeInfo> backup;
 	backup = rangeMap;  // QMap is implicitly shared
 
-	// paragrahps start from 1, ranges from 0
-	if (computeRanges(fromSha, paraFrom - 1, paraTo - 1, toSha).isEmpty())
+	// paragrahps start from 0 but ranges from 1
+	if (computeRanges(fromSha, *paraFrom + 1, *paraTo + 1, toSha).isEmpty())
 		goto fail;
 
 	if (!rangeMap.contains(toSha))
 		goto fail;
 
-	paraFrom = rangeMap[toSha].start + 1;
-	paraTo = rangeMap[toSha].end + 1;
+	*paraFrom = rangeMap[toSha].start - 1;
+	*paraTo = rangeMap[toSha].end - 1;
 	rangeMap = backup;
 	return true;
 
