@@ -11,6 +11,7 @@
 #include <qprocess.h>
 #include <qtimer.h>
 #include <qpair.h>
+#include "exceptionmanager.h"
 #include "common.h"
 
 class Git;
@@ -31,7 +32,7 @@ public:
 	RangeInfo() { clear(); };
 	RangeInfo(int s, int e, bool m) : start(s), end(e), modified(m) {};
 	void clear() { start = end = 0; modified = false; };
-	int start, end;
+	int start, end; // ranges count file lines from 1 like patches diffs
 	bool modified;
 };
 
@@ -39,7 +40,7 @@ class Annotate : public QObject {
 Q_OBJECT
 public:
 	Annotate (Git* parent, QObject* guiObj);
-	bool stop();
+	void deleteWhenDone();
 	const FileAnnotation* lookupAnnotation(SCRef sha, SCRef fileName);
 	bool start(const FileHistory* fh);
 	bool isValid() { return valid; };
@@ -49,13 +50,14 @@ public:
 	QString file() { return fileName; };
 	const QString getAncestor(SCRef sha, SCRef fileName, int* shaIdx);
 	bool getRange(SCRef sha, RangeInfo* r);
-	bool seekPosition(int& rangeStart, int& rangeEnd, SCRef fromSha, SCRef toSha);
-	const QString computeRanges(SCRef sha, int rangeStart,
-	                            int rangeEnd, SCRef target = "");
+	bool seekPosition(int* rangeStart, int* rangeEnd, SCRef fromSha, SCRef toSha);
+	const QString computeRanges(SCRef sha, int rStart, int rEnd, SCRef target = "");
+
 private slots:
 	void on_patchProc_readFromStdout();
 	void on_patchProc_processExited();
 	void on_progressTimer_timeout();
+	void on_deleteWhenDone();
 	void slotComputeDiffs();
 
 private:
@@ -66,8 +68,7 @@ private:
 	FileAnnotation* getFileAnnotation(SCRef sha);
 	void setInitialAnnotation(SCRef fileName, SCRef sha, FileAnnotation* fa);
 	const QString setupAuthor(SCRef origAuthor, int annId);
-	void setAnnotation(SCRef diff, SCRef author,
-	                   SCList prevAnn, QStringList& newAnn, int ofs = 0);
+	void setAnnotation(SCRef diff, SCRef aut, SCList pAnn, QStringList& nAnn, int ofs = 0);
 	bool getNextLine(SCRef d, int& idx, QString& line);
 	void unify(SList dst, SCList src);
 	void updatePatchScript(SCRef sha, SCRef par);
@@ -75,22 +76,24 @@ private:
 	const QString getNextPatch(QString& patchFile, SCRef fileName, SCRef sha);
 	bool getNextSection(SCRef d, int& idx, QString& sec, SCRef target);
 	void updateRange(RangeInfo* r, SCRef diff, bool reverse);
-	void updateCrossRanges(SCRef chunk, bool reverse, int origStart,
-	                       int origLineCnt, RangeInfo* r);
+	void updateCrossRanges(SCRef cnk, bool rev, int oStart, int oLineCnt, RangeInfo* r);
+
+	EM_DECLARE(exAnnCanceled);
 
 	Git* git;
 	QObject* gui;
 	const FileHistory* fh;
 	AnnotateHistory ah;
-	volatile bool cancelingAnnotate;
-	volatile bool annotateRunning;
+	bool cancelingAnnotate;
+	bool annotateRunning;
+	bool annotateActivity;
 	bool isError;
 	int annNumLen;
 	int annId;
 	int annFilesNum;
 	QString patchScript;
 	QString fileName;
-	StrVect histRevOrder;
+	StrVect histRevOrder; // TODO use reference
 	QProcess patchProc;
 	QString patchProcBuf;
 	QString nextFileSha;
