@@ -12,10 +12,10 @@
 class Cache : public QObject {
 Q_OBJECT
 public:
-	Cache(QObject *par = 0) : QObject(par) {};
-	bool save(const QString& workDir, const RevFileMap& rf,
+	explicit Cache(QObject* par) : QObject(par) {};
+	bool save(const QString& gitDir, const RevFileMap& rf,
 	          const StrVect& dirs, const StrVect& files);
-	bool load(const QString& workDir, RevFileMap& rf,
+	bool load(const QString& gitDir, RevFileMap& rf,
 	          StrVect& dirs, StrVect& files);
 };
 
