@@ -21,8 +21,7 @@ class DataLoader : public QObject {
 Q_OBJECT
 public:
 	DataLoader(Git* g, FileHistory* f);
-	~DataLoader();
-	bool start(const QString& cmd, const QString& wd);
+	bool start(const QStringList& args, const QString& wd);
 
 signals:
 	void newDataReady(const FileHistory*);
