@@ -19,7 +19,8 @@
 #define EM_RAISE(x)      ExceptionManager::em()->raise(x)
 #define EM_MATCH(x,y,z)  ExceptionManager::em()->isMatch(x,y,QString(z))
 #define EM_DESC(x)       ExceptionManager::em()->desc(x)
-#define EM_CHECK_PENDING ExceptionManager::em()->throwPending()
+#define EM_IS_PENDING(x) ExceptionManager::em()->isPending(x)
+#define EM_THROW_PENDING ExceptionManager::em()->throwPending()
 
 #define EM_BEFORE_PROCESS_EVENTS int _region = ExceptionManager::em()->saveThrowableSet()
 
@@ -40,7 +41,7 @@ class ExceptionManager { // singleton class
 protected:
 	ExceptionManager();
 	ExceptionManager(const ExceptionManager&);
-	ExceptionManager& operator= (const ExceptionManager&);
+	ExceptionManager& operator=(const ExceptionManager&);
 
 public:
 	static ExceptionManager* em() {
@@ -56,6 +57,7 @@ public:
 	void restoreThrowableSet(int regionId);
 	bool isMatch(int value, int excpId, const QString& context);
 	const QString desc(int excpId);
+	bool isPending(int excpId);
 
 private:
 	int excpId;
