@@ -19,6 +19,7 @@
 #include <qprocess.h>
 #include <qtextcodec.h>
 #include <qstylesheet.h>
+#include <qobjectlist.h>
 #include "mainimpl.h"
 #include "annotate.h"
 #include "cache.h"
@@ -27,14 +28,14 @@
 
 using namespace QGit;
 
-Git::Git(QWidget* p, const char* n) : QObject(p, n), par(p) {
+Git::Git(QWidget* p) : QObject(p) {
 
 	EM_INIT(exGitStopped, "Stopping connection with git");
 
 	cacheNeedsUpdate = isMergeHead = false;
 	isStGIT = isGIT = loadingUnAppliedPatches = false;
 	errorReportingEnabled = true; // report errors if run() fails
-	runningProcesses = 0;
+	curDomain = NULL;
 
 	revs.resize(MAX_DICT_SIZE);
 	revsFiles.resize(MAX_DICT_SIZE);
@@ -42,8 +43,10 @@ Git::Git(QWidget* p, const char* n) : QObject(p, n), par(p) {
 	revsFiles.setAutoDelete(true);
 
 	cache = new Cache(this);
+}
+
+void Git::checkEnvironment() {
 
-	// control git version
 	QString version;
 	if (run("git --version", &version)) {
 
@@ -54,17 +57,40 @@ Git::Git(QWidget* p, const char* n) : QObject(p, n), par(p) {
 			// policy should be implemented upstream
 			const QString cmd("Current git version is " + version +
 			      " but is required " + GIT_VERSION + " or better");
+
 			const QString errorDesc("Your installed git is too old."
 			      "\nPlease upgrade to avoid possible misbehaviours.");
+
 			MainExecErrorEvent* e = new MainExecErrorEvent(cmd, errorDesc);
-			QApplication::postEvent(p, e);
+			QApplication::postEvent(parent(), e);
 		}
 	}
 	errorReportingEnabled = false;
 	isTextHighlighterFound = run("source-highlight -V", &version);
+	errorReportingEnabled = true;
 	if (isTextHighlighterFound)
 		dbp("Found %1", version.section('\n', 0, 0));
-	errorReportingEnabled = true;
+}
+
+void Git::setThrowOnStop(bool b) {
+
+	if (b)
+		EM_REGISTER(exGitStopped);
+	else
+		EM_REMOVE(exGitStopped);
+}
+
+bool Git::isThrowOnStopRaised(int excpId, SCRef curContext) {
+
+	return EM_MATCH(excpId, exGitStopped, curContext);
+}
+
+bool Git::allProcessDeleted() {
+
+	QObjectList* l = this->queryList("QProcess");
+	bool allProcessAreDeleted = (l->count() == 0);
+	delete l;
+	return allProcessAreDeleted;
 }
 
 void Git::setTextCodec(QTextCodec* tc) {
@@ -96,6 +122,18 @@ QTextCodec* Git::getTextCodec(bool* isGitArchive) {
 	return QTextCodec::codecForName(runOutput.stripWhiteSpace());
 }
 
+const QString Git::quote(SCRef nm) {
+
+	return (QUOTE_CHAR + nm + QUOTE_CHAR);
+}
+
+const QString Git::quote(SCList sl) {
+
+	QString q(sl.join(QUOTE_CHAR + ' ' + QUOTE_CHAR));
+	q.prepend(QUOTE_CHAR).append(QUOTE_CHAR);
+	return q;
+}
+
 const QString Git::getLocalDate(SCRef gitDate) {
 
 	QDateTime d;
@@ -220,17 +258,11 @@ Annotate* Git::startAnnotate(FileHistory* fh, QObject* guiObj) { // non blocking
 void Git::cancelAnnotate(Annotate* ann) {
 
 	if (ann)
-		if (ann->stop()) // if still running will be deleted by annotateExited()
-			delete ann;
+		ann->deleteWhenDone();
 }
 
 void Git::annotateExited(Annotate* ann) {
 
-	if (ann->isCanceled()) {
-		ann->deleteLater();
-		return; // do not emit anything
-	}
-	// Annotate object will be deleted by cancelAnnotate()
 	const QString msg = QString("Annotated %1 files in %2 ms")
 	                   .arg(ann->count()).arg(ann->elapsed());
 
@@ -259,13 +291,13 @@ const Rev* Git::revLookup(SCRef sha, const FileHistory* fh) {
 
 bool Git::run(SCRef runCmd, QString* runOutput, QObject* receiver, SCRef buf) {
 
-	MyProcess p(par, this, workDir, errorReportingEnabled);
+	MyProcess p(parent(), this, workDir, errorReportingEnabled);
 	return p.runSync(runCmd, runOutput, receiver, buf);
 }
 
 MyProcess* Git::runAsync(SCRef runCmd, QObject* receiver, SCRef buf) {
 
-	MyProcess* p = new MyProcess(par, this, workDir, errorReportingEnabled);
+	MyProcess* p = new MyProcess(parent(), this, workDir, errorReportingEnabled);
 	if (!p->runAsync(runCmd, receiver, buf)) {
 		delete p;
 		p = NULL;
@@ -408,6 +440,9 @@ const QString Git::getTagMsg(SCRef sha) {
 
 MyProcess* Git::getDiff(SCRef sha, QObject* receiver, SCRef diffToSha, bool combined) {
 
+	if (sha.isEmpty())
+		return NULL;
+
 	QString runCmd;
 	if (sha != ZERO_SHA) {
 		runCmd = "git diff-tree -r --patch-with-stat ";
@@ -422,7 +457,7 @@ MyProcess* Git::getDiff(SCRef sha, QObject* receiver, SCRef diffToSha, bool comb
 const QString Git::getFileSha(SCRef file, SCRef revSha) {
 
 	const QString sha(revSha == ZERO_SHA ? "HEAD" : revSha);
-	QString runCmd("git ls-tree -r " + sha + " " + file), runOutput;
+	QString runCmd("git ls-tree -r " + sha + " " + quote(file)), runOutput;
 	bool ok = run(runCmd, &runOutput);
 	if (!ok || runOutput.isEmpty()) // deleted file case
 		return "";
@@ -434,7 +469,7 @@ MyProcess* Git::getFile(SCRef file, SCRef revSha, QObject* receiver, QString* re
 
 	QString runCmd;
 	if (revSha == ZERO_SHA)
-		runCmd = "cat " + file;
+		runCmd = "cat " + quote(file);
 	else {
 		SCRef fileSha(getFileSha(file, revSha));
 		if (!fileSha.isEmpty())
@@ -460,7 +495,7 @@ MyProcess* Git::getHighlightedFile(SCRef file, SCRef sha, QObject* receiver, QSt
 	if (!saveFile(file, sha, inputFile))
 		return NULL;
 
-	QString runCmd("source-highlight --failsafe -f html -i " + inputFile);
+	QString runCmd("source-highlight --failsafe -f html -i " + quote(inputFile));
 
 	if (receiver == NULL) {
 		run(runCmd, result);
@@ -476,7 +511,7 @@ MyProcess* Git::getHighlightedFile(SCRef file, SCRef sha, QObject* receiver, QSt
 void Git::on_getHighlightedFile_eof() {
 
 	QDir dir(workDir);
-	QStringList sl(dir.entryList("qgit_hlght_input*"));
+	const QStringList sl(dir.entryList("qgit_hlght_input*"));
 	loopList(it, sl)
 		dir.remove(*it);
 }
@@ -488,7 +523,7 @@ bool Git::saveFile(SCRef file, SCRef sha, SCRef path) {
 	return writeToFile(path, fileData);
 }
 
-void Git::getTree(SCRef treeSha, SList names, SList shas,
+bool Git::getTree(SCRef treeSha, SList names, SList shas,
                   SList types, bool isWorkingDir, SCRef treePath) {
 
 	QStringList newFiles, unfiles, delFiles, dummy;
@@ -507,7 +542,7 @@ void Git::getTree(SCRef treeSha, SList names, SList shas,
 	const QString tree(treeSha == ZERO_SHA ? "HEAD" : treeSha);
 	QString runOutput;
 	if (!run("git ls-tree " + tree, &runOutput))
-		return;
+		return false;
 
 	const QStringList sl(QStringList::split('\n', runOutput));
 	loopList(it, sl) {
@@ -534,6 +569,7 @@ void Git::getTree(SCRef treeSha, SList names, SList shas,
 		types.append(NEW);
 		newFiles.pop_front();
 	}
+	return true;
 }
 
 void Git::getWorkDirFiles(const QChar& status, SList files, SList dirs) {
@@ -566,7 +602,7 @@ bool Git::isTreeModified(SCRef sha) {
 	const RevFile* f = getFiles(sha);
 	if (f) {
 		for (uint i = 0; i < f->names.count(); ++i)
-			if (f->statusCmp(i, DELETED) || f->statusCmp(i, NEW))
+			if (!f->statusCmp(i, MODIFIED))
 				return true;
 	}
 	return false;
@@ -822,7 +858,7 @@ const RevFile* Git::getFiles(SCRef sha, SCRef diffToSha, bool allFiles, SCRef pa
 
 void Git::startFileHistory(FileHistory* fh) {
 
-	startRevList(fh->fileName, fh);
+	startRevList(quote(fh->fileName), fh);
 }
 
 void Git::getFileFilter(SCRef path, QMap<QString, bool>& shaMap) {
@@ -858,11 +894,11 @@ bool Git::getPatchFilter(SCRef exp, bool isRegExp, QMap<QString, bool>& shaMap)
 	if (isRegExp)
 		runCmd.append("--pickaxe-regex ");
 
-	runCmd.append(QUOTE_CHAR + "-S" + exp + QUOTE_CHAR);
+	runCmd.append(quote("-S" + exp));
 	if (!run(runCmd, &runOutput, NULL, buf))
 		return false;
 
-	QStringList sl(QStringList::split('\n', runOutput));
+	const QStringList sl(QStringList::split('\n', runOutput));
 	loopList(it2, sl)
 		shaMap.insert(*it2, true);
 
@@ -876,22 +912,22 @@ bool Git::resetCommits(int parentDepth) {
 	return run(runCmd);
 }
 
-bool Git::applyPatchFile(SCRef patchPath, bool commit, bool fold, bool sign) {
-
-	const QString quotedPath(QUOTE_CHAR + patchPath + QUOTE_CHAR);
+bool Git::applyPatchFile(SCRef patchPath, bool commit, bool fold, bool isDragDrop) {
 
 	if (commit && isStGIT) {
 		if (fold)
-			return run("stg fold " + quotedPath);
+			return run("stg fold " + quote(patchPath));
 
-		return run("stg import --mail " + quotedPath);
+		return run("stg import --mail " + quote(patchPath));
 	}
-	QString runCmd("git am -k -u --3way ");
+	QString runCmd("git am --utf8 --3way ");
+	if (isDragDrop)
+		runCmd.append("--keep ");
 
-	if (testFlag(SIGN_PATCH_F) && sign)
+	else if (testFlag(SIGN_PATCH_F))
 		runCmd.append("--signoff ");
 
-	return run(runCmd + quotedPath);
+	return run(runCmd + quote(patchPath));
 }
 
 bool Git::formatPatch(SCList shaList, SCRef dirPath, SCRef remoteDir) {
@@ -917,12 +953,12 @@ bool Git::formatPatch(SCList shaList, SCRef dirPath, SCRef remoteDir) {
 	if (remote)
 		workDir = remoteDir; // run() uses workDir value
 
-	int n = 1;
+	int n = shaList.count();
 	bool ret = false;
 	loopList(it, shaList) { // shaList is ordered by newest to oldest
 		const QString cmd(runCmd + QString::number(n) + " " +
 		                  *it + QString::fromLatin1("^..") + *it);
-		n++;
+		n--;
 		ret = run(cmd);
 		if (!ret)
 			break;
@@ -937,7 +973,7 @@ bool Git::updateIndex(SCList selFiles) {
 		return true;
 
 	QString runCmd("git update-index --add --remove --replace -- ");
-	runCmd.append(selFiles.join(" "));
+	runCmd.append(quote(selFiles));
 	return run(runCmd);
 }
 
@@ -966,7 +1002,7 @@ bool Git::commitFiles(SCList selFiles, SCRef msg) {
 
 	// extract not selected files already updated
 	// in index, save them to restore at the end
-	QStringList notSelInIndexFiles(getOtherFiles(selFiles, optOnlyInIndex));
+	const QStringList notSelInIndexFiles(getOtherFiles(selFiles, optOnlyInIndex));
 
 	// extract selected NOT to be deleted files to
 	// later feed git commit. Files to be deleted
@@ -991,8 +1027,10 @@ bool Git::commitFiles(SCList selFiles, SCRef msg) {
 
 	// now we can commit, 'git commit' will update index
 	// with selected (not to be deleted) files for us
-	QString runCmd("git commit -i" + cmtOptions + " -F " + msgFile);
-	runCmd.append(" " + selNotDelFiles.join(" "));
+	QString runCmd("git commit" + cmtOptions + " -F " + msgFile);
+	if (!selNotDelFiles.empty())
+		runCmd.append(" -i " + quote(selNotDelFiles));
+
 	if (!run(runCmd))
 		return false;
 
@@ -1070,8 +1108,8 @@ bool Git::stgCommit(SCList selFiles, SCRef msg, SCRef patchName, bool fold) {
 	// we have selected modified files in selFiles, we still need
 	// to know the not selected but modified files and, among
 	// theese the cached ones to proper restore state at the end.
-	QStringList notSelFiles = getOtherFiles(selFiles, !optOnlyInIndex);
-	QStringList notSelInIndexFiles = getOtherFiles(selFiles, optOnlyInIndex);
+	const QStringList notSelFiles = getOtherFiles(selFiles, !optOnlyInIndex);
+	const QStringList notSelInIndexFiles = getOtherFiles(selFiles, optOnlyInIndex);
 
 	// update index with selected files
 	if (!run("git read-tree --reset HEAD"))
@@ -1100,12 +1138,12 @@ bool Git::stgCommit(SCList selFiles, SCRef msg, SCRef patchName, bool fold) {
 			if (!run("stg refresh --message \"" + msg.stripWhiteSpace() + "\""))
 				goto rollback;
 		}
-		if (!run("stg fold " + patchFile))
+		if (!run("stg fold " + quote(patchFile)))
 			goto rollback;
 		if (!run("stg refresh")) // refresh needed after fold
 			goto rollback;
 	} else {
-		 if (!run("stg import --mail --name " + patchName + " " + patchFile))
+		if (!run("stg import --mail --name " + quote(patchName) + " " + quote(patchFile)))
 			goto rollback;
 	}
 	goto exit;
@@ -1149,8 +1187,7 @@ bool Git::deleteTag(SCRef sha) {
 
 bool Git::stgPush(SCRef sha) {
 
-	const QString quotedPath(QUOTE_CHAR + patchNames[sha] + QUOTE_CHAR);
-	bool ret = run("stg push " + quotedPath);
+	bool ret = run("stg push " + quote(patchNames[sha]));
 	if (!ret)
 		run("stg push --undo");
 
@@ -1165,8 +1202,7 @@ bool Git::stgPop(SCRef sha) {
 	if (sha == appliedSHA.first())
 		return run("stg pop --all");
 
-	const QString quotedPath(QUOTE_CHAR + patchNames[sha] + QUOTE_CHAR);
-	if (!run("stg pop --to=" + quotedPath))
+	if (!run("stg pop " + quote(patchNames[sha])))
 		return false;
 
 	return run("stg pop");
@@ -1189,7 +1225,7 @@ bool Git::readFromFile(SCRef fileName, QString& data) {
 
 	data = "";
 	QFile file(QFile::encodeName(fileName));
-	if (!file.open(IO_ReadOnly)){
+	if (!file.open(IO_ReadOnly)) {
 		dbp("ERROR: unable to read file %1", fileName);
 		return false;
 	}
