@@ -12,7 +12,6 @@
 #include <qeventloop.h>
 #include <qregexp.h>
 #include <qtextcodec.h>
-#include <qobjectlist.h>
 #include "exceptionmanager.h"
 #include "rangeselectimpl.h"
 #include "cache.h"
@@ -21,7 +20,7 @@
 #include "dataloader.h"
 #include "git.h"
 
-#define POST_MSG(x) QApplication::postEvent(par, new MessageEvent(x))
+#define POST_MSG(x) QApplication::postEvent(parent(), new MessageEvent(x))
 
 using namespace QGit;
 
@@ -44,7 +43,7 @@ const QString Git::getArgs(QString* range, bool askForRange, bool* quit) {
 	    &&  testFlag(RANGE_SELECT_F)
 	    && (!startup || range->isEmpty())) {
 
-		RangeSelectImpl rs(par, range, getTagNames(), quit);
+		RangeSelectImpl rs((QWidget*)parent(), range, getTagNames(), quit);
 		rs.exec(); // modal execution
 		if (*quit)
 			return "";
@@ -75,7 +74,7 @@ const QString Git::getBaseDir(bool* changed, SCRef wd, bool* ok, QString* gd) {
 	}
 	// 'git rev-parse --git-dir' output could be a relative
 	// to working dir (as ex .git) or an absolute path
- 	QDir d(runOutput.startsWith("/") ? runOutput : wd + "/" + runOutput);
+	QDir d(runOutput.startsWith("/") ? runOutput : wd + "/" + runOutput);
 	*changed = (d.absPath() != gitDir);
 	if (gd)
 		*gd = d.absPath();
@@ -113,7 +112,7 @@ bool Git::getRefs() {
 	if (!run("git peek-remote " + gitDir, &runOutput))
 		return false;
 
-	QStringList rLst(QStringList::split('\n', runOutput));
+	const QStringList rLst(QStringList::split('\n', runOutput));
 	loopList(it, rLst) {
 
 		SCRef refSha = (*it).left(40);
@@ -194,7 +193,7 @@ void Git::dirWalker(SCRef dirPath, SList files, SList filesSHA, SCRef nameFilter
 bool Git::lookUpPatchesSHA(SCRef patchName, SList files, SList filesSHA, QString& sha) {
 // this function updates sha and also patchNames map
 
-	QStringList fl(files.grep("/" + patchName + "/top"));
+	const QStringList fl(files.grep("/" + patchName + "/top"));
 	if (fl.count() != 1) {
 		qDebug("ASSERT: found %i patches instead of 1 in %s",
 		       fl.count(), patchName.latin1());
@@ -226,7 +225,7 @@ bool Git::getStGITPatches() {
 		return false; // false is used by caller as early exit
 
 	QStringList ul, al;
-	const QStringList pl = QStringList::split('\n', runOutput);
+	const QStringList pl(QStringList::split('\n', runOutput));
 	loopList(it, pl) { // keep stg order for unapplied
 		SCRef st = (*it).left(1);
 		SCRef pn = (*it).mid(2);
@@ -249,7 +248,7 @@ bool Git::getStGITPatches() {
 		unAppliedSHA.append(sha);
 	}
 	// the same for applied, in this case the order is not important
-	for (it = al.constBegin(); it != al.constEnd(); ++it){
+	for (it = al.constBegin(); it != al.constEnd(); ++it) {
 		if (!lookUpPatchesSHA(*it, files, filesSHA, sha))
 			return false;
 		appliedSHA.append(sha);
@@ -285,10 +284,10 @@ const QStringList Git::getOthersFiles() {
 	if (!exFile.isEmpty()) {
 		QString path = (exFile.startsWith("/")) ? exFile : workDir + "/" + exFile;
 		if (QFile::exists(path))
-			runCmd.append(" --exclude-from=" + exFile);
+			runCmd.append(" --exclude-from=" + quote(exFile));
 	}
 	if (!exPerDir.isEmpty())
-		runCmd.append(" --exclude-per-directory=" + exPerDir);
+		runCmd.append(" --exclude-per-directory=" + quote(exPerDir));
 	QString runOutput;
 	run(runCmd, &runOutput);
 	return QStringList::split('\n', runOutput);
@@ -370,7 +369,7 @@ void Git::parseDiffFormatLine(RevFile& rf, SCRef line, int parNum) {
 
 void Git::setStatus(RevFile& rf, SCRef rowSt, int parNum) {
 
-	QStringList sl = QStringList::split('\t', rowSt);
+	const QStringList sl(QStringList::split('\t', rowSt));
 	if (sl.count() == 3) {
 
 		// we want store extra info with format "origFile --> destFile (Rxx%)"
@@ -427,15 +426,21 @@ bool Git::startParseProc(SCRef initCmd, FileHistory* fh) {
 	        SLOT(on_loaded(const FileHistory*, ulong, int,
 	        bool, const QString&, const QString&)));
 
-	return dl->start(initCmd, workDir);
+	QStringList args;
+	MyProcess::parseArgs(initCmd, args);
+	return dl->start(args, workDir);
 }
 
 bool Git::startRevList(SCRef args, FileHistory* fh) {
 
 	refsStillToFind = refsStillToFindMasterCopy;
-	QString initCmd("git rev-list --header --topo-order --boundary --parents ");
+	QString initCmd("git rev-list --header --boundary --parents ");
 	if (fh)
+		// fetch history from all trees so any revision in
+		// main view that changes the file is always found
 		initCmd.append("--remove-empty --all -- ");
+	else
+		initCmd.append("--topo-order ");
 
 	return startParseProc(initCmd + args, fh);
 }
@@ -455,19 +460,10 @@ bool Git::stop(bool saveCache) {
 
 	EM_RAISE(exGitStopped);
 
-	emit cancelAllProcesses();
-
-	EM_BEFORE_PROCESS_EVENTS;
-
-	// wait until all processes are exited. Note that we
-	// could have been called while in run(), in this case we
-	// exit from stop() while still in run() and corresponding
-	// QProcess is still not deleted
-	while (runningProcesses > 0) {
-		usleep(20000);
-		qApp->processEvents(QEventLoop::ExcludeUserInput);
-	}
-	EM_AFTER_PROCESS_EVENTS;
+	// stop all data sending from process and asks them
+	// to terminate. Note that process could still keep
+	// running for a while although silently
+	emit cancelAllProcesses(); // non blocking
 
 	if (cacheNeedsUpdate && saveCache) {
 		cacheNeedsUpdate = false;
@@ -476,14 +472,12 @@ bool Git::stop(bool saveCache) {
 
 		if (!revsFiles.isEmpty()) {
 			POST_MSG("Saving cache. Please wait...");
-			if (!cache->save(workDir, revsFiles, dirNamesVec, fileNamesVec))
-				qDebug("ERROR unable to update cache");
+			EM_PROCESS_EVENTS_NO_INPUT; // to paint the message
+			if (!cache->save(gitDir, revsFiles, dirNamesVec, fileNamesVec))
+				dbs("ERROR unable to update cache");
 		}
 	}
-	QObjectList* l = this->queryList("QProcess");
-	bool allDeleted = (l->count() == 0);
-	delete l;
-	return allDeleted;
+	return allProcessDeleted();
 }
 
 void Git::clearRevs() {
@@ -516,12 +510,11 @@ bool Git::init(SCRef wd, bool askForRange, QStringList* filterList, bool* quit)
 	bool filteredLoading = (filterList != NULL);
 	clearRevs();
 	try {
-		EM_REGISTER(exGitStopped);
-
+		setThrowOnStop(true);
 		bool archiveChanged;
 		workDir = getBaseDir(&archiveChanged, wd, &isGIT, &gitDir);
 		if (!isGIT) {
-			EM_REMOVE(exGitStopped);
+			setThrowOnStop(false);
 			return false;
 		}
 		if (archiveChanged) {
@@ -530,8 +523,8 @@ bool Git::init(SCRef wd, bool askForRange, QStringList* filterList, bool* quit)
 			dirNames.clear();
 			fileNamesVec.clear();
 			dirNamesVec.clear();
-			if (!cache->load(workDir, revsFiles, dirNamesVec, fileNamesVec))
-				qDebug("ERROR: unable to load cache file");
+			if (!cache->load(gitDir, revsFiles, dirNamesVec, fileNamesVec))
+				dbs("ERROR: unable to load cache file");
 			else
 				populateFileDict();
 		}
@@ -540,12 +533,12 @@ bool Git::init(SCRef wd, bool askForRange, QStringList* filterList, bool* quit)
 		if (!filteredLoading) {
 			POST_MSG(msg1 + "refs...");
 			if (!getRefs())
-				qDebug("WARNING: no tags or heads found");
+				dbs("WARNING: no tags or heads found");
 
 			POST_MSG("");
 			args = getArgs(&curRange, askForRange, quit); // must be called with refs loaded
 			if (*quit) {
-				EM_REMOVE(exGitStopped);
+				setThrowOnStop(false);
 				return false;
 			}
 			// update text codec according to repo settings
@@ -562,16 +555,14 @@ bool Git::init(SCRef wd, bool askForRange, QStringList* filterList, bool* quit)
 				patchesStillToFind = appliedSHA.count();
 				loadingUnAppliedPatches = true;
 				if (startUnappliedList()) {
-					EM_BEFORE_PROCESS_EVENTS;
 					while (loadingUnAppliedPatches) {
 						// WARNING we are in setRepository()
 						usleep(20000);
-						qApp->processEvents();
+						EM_PROCESS_EVENTS;
 					}
-					EM_AFTER_PROCESS_EVENTS;
 					lns.clear(); // again to reset lanes
 				} else
-					qDebug("ERROR: unable to load unapplied list");
+					dbs("ERROR: unable to load unapplied list");
 			}
 		}
 		if (testFlag(DIFF_INDEX_F) && !filteredLoading) {
@@ -580,23 +571,23 @@ bool Git::init(SCRef wd, bool askForRange, QStringList* filterList, bool* quit)
 		}
 		POST_MSG(msg1 + "revisions...");
 		if (!startRevList(args))
-			qDebug("ERROR: unable to start rev list loading");
+			dbs("ERROR: unable to start rev list loading");
 
-		EM_REMOVE(exGitStopped);
+		setThrowOnStop(false);
 		return true;
 
 	} catch(int i) {
 
-		EM_REMOVE(exGitStopped);
+		setThrowOnStop(false);
 
-		if (EM_MATCH(i, exGitStopped, "initializing")) {
-			EM_CHECK_PENDING;
+		if (isThrowOnStopRaised(i, "initializing")) {
+			EM_THROW_PENDING;
 			return false;
 		}
 		const QString info("Exception \'" + EM_DESC(i) + "\' "
 		                   "not handled in init...re-throw");
-		dbp("%1", info);
-		throw i;
+		dbs(info);
+		throw;
 	}
 }
 
@@ -610,7 +601,7 @@ void Git::on_loaded(const FileHistory* fh, ulong byteSize, int loadTime,
 
 	if (!errorDesc.isEmpty()) {
 		MainExecErrorEvent* e = new MainExecErrorEvent(cmd, errorDesc);
-		QApplication::postEvent(par, e);
+		QApplication::postEvent(parent(), e);
 	}
 	if (normalExit) { // do not send anything if killed
 
@@ -706,7 +697,7 @@ void Git::copyDiffIndex(FileHistory* fh) {
 // in histRevs that will be used as the parent.
 
 	if (!fh->revOrder.isEmpty() || (fh->revs.count() != 1)) {
-		qDebug("ASSERT in copyDiffIndex: called with wrong context");
+		dbs("ASSERT in copyDiffIndex: called with wrong context");
 		return;
 	}
 	const Rev* r = revLookup(ZERO_SHA);
@@ -784,12 +775,12 @@ void Git::updateLanes(Rev& c, Lanes& lns) {
 	if (lns.isBranch())
 		lns.afterBranch();
 
-// 	QString tmp = "", tmp2;
-// 	for (uint i = 0; i < c.lanes.count(); i++) {
-// 		tmp2.setNum(c.lanes[i]);
-// 		tmp.append(tmp2 + "-");
-// 	}
-// 	qDebug("%s %s",tmp.latin1(), c.sha.latin1());
+//	QString tmp = "", tmp2;
+//	for (uint i = 0; i < c.lanes.count(); i++) {
+//		tmp2.setNum(c.lanes[i]);
+//		tmp.append(tmp2 + "-");
+//	}
+//	qDebug("%s %s",tmp.latin1(), c.sha.latin1());
 }
 
 void Git::on_procDataReady(const QString& fileChunk) {
@@ -1062,7 +1053,7 @@ void Git::indexTree() {
 		}
 	}
 	// walk backward through the tree and compute nearest tagged ancestors
-	for (int i = revOrder.count() - 1; i >= 0 ; i--) {
+	for (int i = revOrder.count() - 1; i >= 0; i--) {
 
 		const Rev* r = revLookup(revOrder[i]);
 
