@@ -6,7 +6,6 @@
 	Copyright: See COPYING file that comes with this distribution
 
 */
-#include <qtimer.h>
 #include <qlineedit.h>
 #include <qtextbrowser.h>
 #include <qlistview.h>
@@ -34,6 +33,7 @@
 #include <qtabwidget.h>
 #include <qobjectlist.h>
 #include <qlayout.h>
+#include <qtooltip.h>
 #include "config.h" // defines PACKAGE_VERSION
 #include "help.h"
 #include "helpbase.h"
@@ -56,39 +56,24 @@
 
 using namespace QGit;
 
-// *************** QGit namespace definitions ****************************
+class MyPushButton : public QPushButton {
+public:
+	MyPushButton(const QIconSet& i, SCRef t, QWidget* p) : QPushButton(i, t, p) {}
 
-QColor QGit::ODD_LINE_COL; // declared as extern in QGit
-QColor QGit::EVEN_LINE_COL;
+	// we need this kludge to workaround 'close tab' button wrong width
+	// under some themes as Bluecurve or Platinum. What happens is that
+	// widget sizeHint() returns a wrong value that is used by Qt to paint
+	// the button.
+	// Becuase it is not possible to set sizeHint property directly,
+	// we need to sub class QPushButton and override this puppy
+	virtual QSize sizeHint() const {
 
-uint QGit::flags(SCRef group) {
+		return iconSet()->iconSize(QIconSet::Small);
+	}
+};
 
-	QSettings settings;
-	return settings.readNumEntry(APP_KEY + group + FLAGS_KEY, FLAGS_DEF);
-}
+MainImpl::MainImpl(SCRef cd, QWidget* p) : MainBase(p, "", Qt::WDestructiveClose) {
 
-bool QGit::testFlag(uint f, SCRef group) { return (flags(group) & f); }
-
-void QGit::setFlag(uint f, bool b, SCRef group) {
-
-	QSettings settings;
-	int flags = settings.readNumEntry(APP_KEY + group + FLAGS_KEY, FLAGS_DEF);
-	flags = (b) ? flags | f : flags & ~f;
-	settings.writeEntry(APP_KEY + group + FLAGS_KEY, flags);
-}
-
-void QGit::writeSetting(SCRef key, SCRef value, SCRef group) {
-
-	QSettings settings;
-	settings.writeEntry(APP_KEY + group + key, value);
-}
-
-// ******************************* MainImpl ******************************
-
-MainImpl::MainImpl(const QString& cd, QWidget* p, const char *name) :
-          MainBase(p, name, Qt::WDestructiveClose) {
-
-	EM_INIT(exSetRepositoryCalled, "Reloading data");
 	EM_INIT(exExiting, "Exiting");
 
 	git = new Git(this);
@@ -97,7 +82,7 @@ MainImpl::MainImpl(const QString& cd, QWidget* p, const char *name) :
 	qApp->installEventFilter(this);
 
 	// init native types
-	 setRepositoryBusy = changesCommitted = false;
+	setRepositoryBusy = false;
 
 	// init filter match highlighters
 	shortLogRE.setMinimal(true);
@@ -105,11 +90,29 @@ MainImpl::MainImpl(const QString& cd, QWidget* p, const char *name) :
 	longLogRE.setMinimal(true);
 	longLogRE.setCaseSensitive(false);
 
+	// set-up typewriter (fixed width) font
+	QSettings set;
+	QString font(set.readEntry(APP_KEY + FONT_KEY));
+	if (font.isEmpty()) { // choose a sensible default
+		QFont fnt = QApplication::font();
+		fnt.setStyleHint(QFont::TypeWriter, QFont::PreferDefault);
+		fnt.setFixedPitch(true);
+		fnt.setFamily(fnt.defaultFamily()); // the family corresponding
+		font = fnt.toString();              // to current style hint
+	}
+	QGit::TYPE_WRITER_FONT.fromString(font);
+
 	// set-up tab view
 	delete tabWdg->currentPage(); // cannot be done in Qt Designer
 	rv = new RevsView(this, git);
-	pushButtonCloseTab->reparent(tabWdg, QPoint());
-	tabWdg->setCornerWidget(pushButtonCloseTab);
+
+	// set-up tab corner widget ('close tab' button)
+	pbCloseTabBase->hide();
+	MyPushButton* pb = new MyPushButton(*pbCloseTabBase->iconSet(), "", tabWdg);
+	QToolTip::add(pb, "Close tab");
+	tabWdg->setCornerWidget(pb);
+	connect(pb, SIGNAL(clicked()), this, SLOT(pushButtonCloseTab_clicked()));
+	connect(this, SIGNAL(closeTabButtonEnabled(bool)), pb, SLOT(setEnabled(bool)));
 
 	// set-up tree view
 	treeView->hide();
@@ -123,8 +126,7 @@ MainImpl::MainImpl(const QString& cd, QWidget* p, const char *name) :
 
 	// set-up menu for custom actions
 	connect(Actions, SIGNAL(activated(int)), this, SLOT(on_customAction_activated(int)));
-	QSettings set;
-	QStringList sl = QStringList::split(",", set.readEntry(APP_KEY + MCR_LIST_KEY, ""));
+	QStringList sl(QStringList::split(",", set.readEntry(APP_KEY + MCR_LIST_KEY, "")));
 	doUpdateCustomActionMenu(sl);
 
 	// create light and dark colors for alternate background
@@ -139,9 +141,6 @@ MainImpl::MainImpl(const QString& cd, QWidget* p, const char *name) :
 	int wd = lineEditSHA->fontMetrics().boundingRect(tmp).width();
 	lineEditSHA->setMinimumWidth(wd);
 
-	pixmaps.setAutoDelete(true);
-	adjustFontSize(0); // default is system-wide setting
-
 	connect(git, SIGNAL(newRevsAdded(const FileHistory*, const QValueVector<QString>&)),
 	        this, SLOT(on_newRevsAdded(const FileHistory*, const QValueVector<QString>&)));
 
@@ -163,7 +162,9 @@ MainImpl::MainImpl(const QString& cd, QWidget* p, const char *name) :
 
 void MainImpl::initWithEventLoopActive() {
 
+	git->checkEnvironment();
 	setRepository(startUpDir, false, false);
+	startUpDir = ""; // one shot
 }
 
 void MainImpl::lineEditSHA_returnPressed() {
@@ -239,25 +240,24 @@ void MainImpl::getExternalDiffArgs(QStringList* args) {
 	args->append(fName1);
 }
 
-// *************************** Repo open or changed *************************
+// ********************** Repository open or changed *************************
 
 void MainImpl::setRepository(SCRef newDir, bool refresh, bool keepSelection,
                              QStringList* filterList) {
 
-	if (refresh && ActFilterTree->isOn() && filterList == NULL) {
-		// refresh called while in filtered view
-		ActFilterTree->toggle(); // just remove the filter and reload
-		return;
-	}
 	// Git::stop() and Git::init() are not re-entrant and must not
 	// be active at the same time. Because Git::init calls processEvents()
-	// we need to guard against reentrancy here.
+	// we need to guard against reentrancy here
 	if (setRepositoryBusy)
 		return;
 
 	setRepositoryBusy = true;
 
-	EM_RAISE(exSetRepositoryCalled);
+	// check for a refresh or open of a new repository while in filtered view
+	if (ActFilterTree->isOn() && filterList == NULL)
+		// toggle() triggers a refresh and a following setRepository()
+		// call that is filtered out by setRepositoryBusy guard flag
+		ActFilterTree->toggle(); // triggers ActFilterTree_toggled()
 
 	try {
 		EM_REGISTER(exExiting);
@@ -265,8 +265,17 @@ void MainImpl::setRepository(SCRef newDir, bool refresh, bool keepSelection,
 		bool archiveChanged;
 		curDir = git->getBaseDir(&archiveChanged, newDir);
 
-		git->stop(archiveChanged); // this stops all pending processing
+		if (!git->stop(archiveChanged)) { // stop all pending processes, non blocking
 
+			// some process is still running, schedule a deferred
+			// call and return, waiting for end of activity
+			setRepositoryBusy = false;
+			EM_REMOVE(exExiting);
+
+			// this object will delete itself when done
+			new setRepoDelayed(this, newDir, refresh, keepSelection, filterList);
+			return;
+		}
 		if (archiveChanged && refresh)
 			dbs("ASSERT in setRepository: different dir with no range select");
 
@@ -281,18 +290,8 @@ void MainImpl::setRepository(SCRef newDir, bool refresh, bool keepSelection,
 		updateContextActions("", "", false, false);
 		ActCommit_setEnabled(false);
 
-		int adj = -1; // hack to correctly map col numbers in main view
-
-		if (testFlag(REL_DATE_F)) {
-			secs = QDateTime::currentDateTime().toTime_t();
-			rv->tab()->listViewLog->setColumnText(TIME_COL + adj, "Last Change");
-		} else {
-			secs = 0;
-			rv->tab()->listViewLog->setColumnText(TIME_COL + adj, "Author Date");
-		}
 		if (ActFilterTree->isOn())
-			setCaption(caption() + " - FILTER ON < " +
-			           filterList->join(" ") + " >");
+			setCaption(caption() + " - FILTER ON < " + filterList->join(" ") + " >");
 
 		// tree name should be set before init because in case of
 		// StGIT archives the first revs are sent before init returns
@@ -300,42 +299,38 @@ void MainImpl::setRepository(SCRef newDir, bool refresh, bool keepSelection,
 		rv->treeView->setTreeName(n.prepend('/').section('/', -1, -1));
 
 		bool quit;
-		bool ret = git->init(curDir, !refresh, filterList, &quit);
+		bool ok = git->init(curDir, !refresh, filterList, &quit); // blocking call
 		if (quit)
 			goto exit;
 
-		if (!ret)
+		updateCommitMenu(ok && git->isStGITStack());
+		ActCheckWorkDir->setOn(testFlag(DIFF_INDEX_F)); // could be changed in Git::init()
+
+		if (ok) {
+			updateGlobalActions(true);
+			if (archiveChanged)
+				updateRecentRepoMenu(curDir);
+		} else
 			statusBar()->message("Not a git archive");
 
-		if (ret && archiveChanged)
-			updateRecentRepoMenu(curDir);
-
-		updateCommitMenu(ret && git->isStGITStack());
-
-		// re-enable global actions
-		ActCheckWorkDir->setOn(testFlag(DIFF_INDEX_F));
-		updateGlobalActions(ret);
-
 exit:
 		setRepositoryBusy = false;
 		EM_REMOVE(exExiting);
 
 		if (quit && !startUpDir.isEmpty())
-			ActClose->activate();
-
-		startUpDir = ""; // one shot
+			close();
 
 	} catch (int i) {
 		EM_REMOVE(exExiting);
 
 		if (EM_MATCH(i, exExiting, "loading repository")) {
-			EM_CHECK_PENDING;
+			EM_THROW_PENDING;
 			return;
 		}
-		const QString info("Exception \'" + EM_DESC(i) + "\' "
-		                   "not handled in setRepository...re-throw");
-		dbp("%1", info.latin1());
-		throw i;
+		const QString info("Exception \'" + EM_DESC(i) + "\' not "
+		                   "handled in setRepository...re-throw");
+		dbs(info);
+		throw;
 	}
 }
 
@@ -415,17 +410,17 @@ void MainImpl::on_treeView_doubleClicked(QListViewItem* item) {
 void MainImpl::pushButtonCloseTab_clicked() {
 
 	int curPos = tabWdg->currentPageIndex();
-	QObject* t;
+	Domain* t;
 	switch (currentTabType(&t)) {
 	case TAB_REV:
 		break;
 	case TAB_PATCH:
-		delete t;
+		t->deleteWhenDone();
 		emit tabClosed(curPos);
 		ActViewDiffNewTab->setEnabled(ActViewDiff->isEnabled() && firstTab<PatchView>());
 		break;
 	case TAB_FILE:
-		delete t;
+		t->deleteWhenDone();
 		emit tabClosed(curPos);
 		ActViewFileNewTab->setEnabled(ActViewFile->isEnabled() && firstTab<FileView>());
 		break;
@@ -437,9 +432,9 @@ void MainImpl::pushButtonCloseTab_clicked() {
 
 void MainImpl::ActViewRev_activated() {
 
-	QObject* t;
+	Domain* t;
 	if (currentTabType(&t) == TAB_FILE) {
-		rv->st = (static_cast<FileView*>(t))->st;
+		rv->st = t->st;
 		UPDATE_DOMAIN(rv);
 	}
 	tabWdg->setCurrentPage(rv->tabPos());
@@ -474,9 +469,9 @@ void MainImpl::openFileTab(FileView* fv) {
 
 void MainImpl::ActViewDiff_activated() {
 
-	QObject* t;
+	Domain* t;
 	if (currentTabType(&t) == TAB_FILE) {
-		rv->st = (static_cast<FileView*>(t))->st;
+		rv->st = t->st;
 		UPDATE_DOMAIN(rv);
 	}
 	rv->viewPatch(false);
@@ -491,6 +486,7 @@ void MainImpl::ActViewDiffNewTab_activated() {
 bool MainImpl::eventFilter(QObject* obj, QEvent* ev) {
 
 	if (ev->type() == QEvent::Wheel) {
+
 		QWheelEvent* e = static_cast<QWheelEvent*>(ev);
 		if (e->state() == Qt::AltButton) {
 
@@ -522,7 +518,8 @@ void MainImpl::on_newRevsAdded(const FileHistory* fh, const QValueVector<QString
 
 	// first rev could be a StGIT unapplied patch so check more then once
 	if (   (!git->isNothingToCommit() || git->isUnknownFiles())
-	    && !ActCommit->isEnabled() && !git->isCommittingMerge())
+	    && !ActCommit->isEnabled()
+	    && !git->isCommittingMerge())
 		ActCommit_setEnabled(true);
 }
 
@@ -579,7 +576,7 @@ void MainImpl::filterList(bool isOn, bool onlyHighlight) {
 		case 6:
 			colNum = SHA_MAP_COL;
 			QApplication::setOverrideCursor(QCursor(Qt::WaitCursor));
-			EM_PROCESS_EVENTS; // to let paint wait cursor
+			EM_PROCESS_EVENTS; // to paint wait cursor
 			if (idx == 4)
 				git->getFileFilter(filter, shaMap);
 			else {
@@ -596,13 +593,15 @@ void MainImpl::filterList(bool isOn, bool onlyHighlight) {
 		}
 	} else {
 		patchNeedsUpdate = ((idx == 5) || (idx == 6));
-		descNeedsUpdate = (!shortLogRE.isEmpty() || !longLogRE.isEmpty());
+		descNeedsUpdate = !(shortLogRE.isEmpty() && longLogRE.isEmpty());
 		shortLogRE.setPattern("");
 		longLogRE.setPattern("");
 	}
 	bool evenLine = false;
 	int visibleCnt = 0;
-	QListViewItemIterator it(rv->tab()->listViewLog);
+	QListViewItem* firstItem = NULL;
+	QListView* lv = rv->tab()->listViewLog;
+	QListViewItemIterator it(lv);
 	while (it.current()) {
 		ListViewItem* item = static_cast<ListViewItem*>(it.current());
 		if (isOn) {
@@ -614,8 +613,11 @@ void MainImpl::filterList(bool isOn, bool onlyHighlight) {
 					evenLine = !evenLine;
 				}
 				visibleCnt++;
+				if (visibleCnt == 1) // only once
+					firstItem = item;
+
 			} else if (!onlyHighlight)
-				item->setVisible(false);
+				item->setVisible(false); // does not change listView current item
 		} else {
 			item->setHighlighted(false);
 			item->setEven(evenLine);
@@ -625,24 +627,28 @@ void MainImpl::filterList(bool isOn, bool onlyHighlight) {
 		}
 		++it;
 	}
-	if (descNeedsUpdate) {
-		SCRef d(git->getDesc(rv->st.sha(), shortLogRE, longLogRE));
-		rv->tab()->textBrowserDesc->setText(d);
-	}
+	lv->triggerUpdate(); // for onlyHighlight case
+
+	// set new selection, could be NULL
+	QListViewItem* curItem = lv->currentItem();
+	QListViewItem* newItem = (curItem && curItem->isVisible()) ? curItem : firstItem;
+	rv->st.setSha(newItem ? ((ListViewItem*)newItem)->sha() : "");
+	UPDATE_DOMAIN(rv);
+
+	// if current item does not change we need to force an update
+	if (descNeedsUpdate && (newItem == curItem))
+		emit updateRevDesc();
+
 	if (patchNeedsUpdate)
 		emit highlightPatch(isOn ? filter : "", isRegExp);
 
-	if (isOn) {
-		const QString tmp(QString("Found %1 matches. Toggle filter/highlight "
-		                          "button to remove the filter").arg(visibleCnt));
-		statusBar()->message(tmp);
-	} else {
-		QListView* lv = rv->tab()->listViewLog;
-		lv->ensureItemVisible(lv->currentItem());
-		statusBar()->clear();
-	}
-	scrollListView(1); // hack to force a repaint
-	scrollListView(-1);
+	QString msg;
+	if (isOn)
+		msg = QString("Found %1 matches. Toggle filter/highlight "
+		      "button to remove the filter").arg(visibleCnt);
+
+	// deferred message, let update first
+	QApplication::postEvent(rv, new MessageEvent(msg));
 }
 
 bool MainImpl::passFilter(ListViewItem* item, SCRef filter, int colNum,
@@ -650,21 +656,24 @@ bool MainImpl::passFilter(ListViewItem* item, SCRef filter, int colNum,
 
 	if (colNum == SHA_MAP_COL)
 		// in this case shaMap contains all good sha to search for
-		return shaMap.contains(item->text(COMMIT_COL));
+		return shaMap.contains(item->sha());
 
 	QString field;
-	if (colNum != LOG_MSG_COL) {
-		int adj = (colNum != COMMIT_COL) ? -1 : 0;
+	if (colNum != LOG_MSG_COL && colNum != COMMIT_COL) {
+		int adj = -1;
 		field = item->text(colNum + adj);
 	}
-	if (field.isEmpty()) { // still not setup or colNum == LOG_MSG_COL
-		const Rev& c = *git->revLookup(item->text(COMMIT_COL));
+	if (field.isEmpty()) { // still not setup or colNum is a dummy one
+
+		const Rev* c = git->revLookup(item->sha());
 		if (colNum == LOG_COL)
-			field = c.shortLog();
+			field = c->shortLog();
 		else if (colNum == AUTH_COL)
-			field = c.author();
+			field = c->author();
 		else if (colNum == LOG_MSG_COL)
-			field = c.longLog();
+			field = c->longLog();
+		else if (colNum == COMMIT_COL)
+			field = item->sha();
 	}
 	// wildcard search, case insensitive
 	return (field.find(QRegExp(filter, false, true)) != -1);
@@ -677,7 +686,7 @@ void MainImpl::customEvent(QCustomEvent* e) {
 		MainBase::customEvent(e);
 		return;
 	}
-	SCRef data = de->data();
+	SCRef data = de->myData();
 
 	switch (e->type()) {
 	case ERROR_EV: {
@@ -700,12 +709,12 @@ void MainImpl::customEvent(QCustomEvent* e) {
 		doFileContexPopup(data, e->type());
 		break;
 	default:
-		qDebug("ASSERT in MainImpl::customEvent unhandled event %i", e->type());
+		dbp("ASSERT in MainImpl::customEvent unhandled event %1", e->type());
 		break;
 	}
 }
 
-int MainImpl::currentTabType(QObject** t) {
+int MainImpl::currentTabType(Domain** t) {
 
 	*t = NULL;
 	int curPos = tabWdg->currentPageIndex();
@@ -775,19 +784,19 @@ void MainImpl::tabWdg_currentChanged(QWidget* w) {
 		return;
 
 	// set correct focus for keyboard browsing
-	QObject* t;
+	Domain* t;
 	switch (currentTabType(&t)) {
 	case TAB_REV:
 		static_cast<RevsView*>(t)->tab()->listViewLog->setFocus();
-		pushButtonCloseTab->setEnabled(false);
+		emit closeTabButtonEnabled(false);
 		break;
 	case TAB_PATCH:
 		static_cast<PatchView*>(t)->tab()->textEditDiff->setFocus();
-		pushButtonCloseTab->setEnabled(true);
+		emit closeTabButtonEnabled(true);
 		break;
 	case TAB_FILE:
 		static_cast<FileView*>(t)->tab()->histListView->setFocus();
-		pushButtonCloseTab->setEnabled(true);
+		emit closeTabButtonEnabled(true);
 		break;
 	default:
 		dbs("ASSERT in tabWdg_currentChanged: unknown current page");
@@ -906,7 +915,7 @@ void MainImpl::goMatch(int delta) {
 QTextEdit* MainImpl::getCurrentTextEdit() {
 
 	QTextEdit* te = NULL;
-	QObject* t;
+	Domain* t;
 	switch (currentTabType(&t)) {
 	case TAB_REV:
 		te = static_cast<RevsView*>(t)->tab()->textBrowserDesc;
@@ -941,7 +950,7 @@ void MainImpl::scrollTextEdit(int delta) {
 void MainImpl::scrollListView(int delta) {
 
 	QWidget* lv = NULL;
-	QObject* t;
+	Domain* t;
 	switch (currentTabType(&t)) {
 	case TAB_REV:
 		lv = static_cast<RevsView*>(t)->tab()->listViewLog;
@@ -964,26 +973,14 @@ void MainImpl::scrollListView(int delta) {
 }
 
 void MainImpl::adjustFontSize(int delta) {
-// font size is adjusted on a 'per instance' base and only on list views
+// font size is changed on a 'per instance' base and only on list views
 
-	QListView* lv = rv->tab()->listViewLog; // we cannot use a different list view. Wrong sizes!
-	QFont f(lv->font());
-	f.setPointSize(f.pointSize() + delta);
+	int ps = listViewFont.pointSize() + delta;
+	if (ps < 2)
+		return;
 
-	// little hack to read new item height
-	lv->setFont(f);
-	QListViewItem* item = new QListViewItem(lv);
-	int h = item->height();
-	delete item;
-
-	setupPixmaps(h); // (re)create the pixmaps
-
-	// clear main view graphs
-	rv->listViewLog->repaintAll(f);
-	QPtrList<FileView>* l = getTabs<FileView>();
-	for (FileView* item = l->first(); item; item = l->next())
-		item->histListView->repaintAll(f);
-	delete l;
+	listViewFont.setPointSize(ps);
+	emit repaintListViews(listViewFont);
 }
 
 // ****************************** Menu *********************************
@@ -1038,7 +1035,7 @@ void MainImpl::doUpdateRecentRepoMenu(SCRef newEntry) {
 	File->insertSeparator();
 
 	QStringList::const_iterator it2 = recents.constBegin();
-	for (int i = 1 ; it2 != recents.constEnd() && i <= MAX_RECENT_REPOS; ++it2, ++i)
+	for (int i = 1; it2 != recents.constEnd() && i <= MAX_RECENT_REPOS; ++it2, ++i)
 		File->insertItem(QString::number(i) + " " + *it2);
 
 	for (int i = recents.count() - MAX_RECENT_REPOS; i > 0; i--)
@@ -1058,16 +1055,18 @@ void MainImpl::doContexPopup(SCRef sha) {
 	connect(contextMenu, SIGNAL(activated(int)), this, SLOT(on_goRef_activated(int)));
 	connect(contextSubMenu, SIGNAL(activated(int)), this, SLOT(on_goRef_activated(int)));
 
-	QObject* t;
+	Domain* t;
 	int tt = currentTabType(&t);
 	bool isRevPage = (tt == TAB_REV);
 	bool isPatchPage = (tt == TAB_PATCH);
 	bool isFilePage = (tt == TAB_FILE);
 
-	if (!isFilePage && ActCheckWorkDir->isEnabled())
+	if (!isFilePage && ActCheckWorkDir->isEnabled()) {
 		ActCheckWorkDir->addTo(contextMenu);
-
-	contextMenu->insertSeparator();
+		contextMenu->insertSeparator();
+	}
+	if (isFilePage && ActViewRev->isEnabled())
+		ActViewRev->addTo(contextMenu);
 
 	if (!isPatchPage && ActViewDiff->isEnabled())
 		ActViewDiff->addTo(contextMenu);
@@ -1075,9 +1074,6 @@ void MainImpl::doContexPopup(SCRef sha) {
 	if (isRevPage && ActViewDiffNewTab->isEnabled())
 		ActViewDiffNewTab->addTo(contextMenu);
 
-	if (isFilePage && ActViewRev->isEnabled())
-		ActViewRev->addTo(contextMenu);
-
 	if (!isFilePage && ActExternalDiff->isEnabled())
 		ActExternalDiff->addTo(contextMenu);
 
@@ -1136,7 +1132,7 @@ void MainImpl::doFileContexPopup(SCRef fileName, int type) {
 
 	QPopupMenu contextMenu;
 
-	QObject* t;
+	Domain* t;
 	int tt = currentTabType(&t);
 	bool isRevPage = (tt == TAB_REV);
 	bool isPatchPage = (tt == TAB_PATCH);
@@ -1180,7 +1176,7 @@ void MainImpl::on_goRef_activated(int id) {
 void MainImpl::ActSplitView_activated() {
 
 	bool hide;
-	QObject* t;
+	Domain* t;
 	switch (currentTabType(&t)) {
 	case TAB_REV: {
 		RevsView* rv = static_cast<RevsView*>(t);
@@ -1343,7 +1339,7 @@ void MainImpl::ActMailApplyPatch_activated() {
 
 	QApplication::setOverrideCursor(QCursor(Qt::WaitCursor));
 
-	if (git->applyPatchFile(f.absFilePath(), commit, fold, Git::optSign) && !commit)
+	if (git->applyPatchFile(f.absFilePath(), commit, fold, !Git::optDragDrop) && !commit)
 		git->resetCommits(1);
 
 	QApplication::restoreOverrideCursor();
@@ -1363,7 +1359,7 @@ void MainImpl::ActCheckWorkDir_toggled(bool b) {
 void MainImpl::ActSettings_activated() {
 
 	SettingsImpl* setView = new SettingsImpl(this, git);
-	setView->exec(); // modal exec
+	setView->exec(); // modal exec, has Qt::WDestructiveClose
 
 	// update ActCheckWorkDir if necessary
 	if (ActCheckWorkDir->isOn() != testFlag(DIFF_INDEX_F))
@@ -1372,8 +1368,10 @@ void MainImpl::ActSettings_activated() {
 
 void MainImpl::ActCustomActionSetup_activated() {
 
-	CustomActionImpl* ca = new CustomActionImpl(); // has WDestructiveClose
+	CustomActionImpl* ca = new CustomActionImpl(); // has Qt::WDestructiveClose
+
 	connect(this, SIGNAL(closeAllWindows()), ca, SLOT(close()));
+
 	connect(ca, SIGNAL(listChanged(const QStringList&)),
 	        this, SLOT(on_customActionListChanged(const QStringList&)));
 
@@ -1391,7 +1389,7 @@ void MainImpl::on_customActionListChanged(const QStringList& list) {
 			w->doUpdateCustomActionMenu(list);
 		++it;
 	}
-	delete l; // list is created using new by topLevelWidgets()
+	delete l; // needs to be deleted, see docs
 }
 
 void MainImpl::doUpdateCustomActionMenu(const QStringList& list) {
@@ -1414,7 +1412,7 @@ void MainImpl::on_customAction_activated(int id) {
 		return;
 
 	QSettings set;
-	QStringList sl = QStringList::split(",", set.readEntry(APP_KEY + MCR_LIST_KEY, ""));
+	const QStringList sl(QStringList::split(",", set.readEntry(APP_KEY + MCR_LIST_KEY, "")));
 
 	if (sl.findIndex(name) == -1) {
 		dbp("ASSERT in on_customAction_activated, action %1 not found", name);
@@ -1435,10 +1433,12 @@ void MainImpl::on_customAction_activated(int id) {
 	if (cmd.isEmpty())
 		return;
 
-	ConsoleImpl* c = new ConsoleImpl(name, git); // has WDestructiveClose
+	ConsoleImpl* c = new ConsoleImpl(name, git); // has Qt::WDestructiveClose
+
 	connect(this, SIGNAL(closeAllWindows()), c, SLOT(close()));
+
 	connect(c, SIGNAL(customAction_exited(const QString&)),
-		this, SLOT(on_customAction_exited(const QString&)));
+	        this, SLOT(on_customAction_exited(const QString&)));
 
 	if (c->start(cmd, cmdArgs))
 		c->show();
@@ -1453,11 +1453,19 @@ void MainImpl::on_customAction_exited(const QString& name) {
 
 void MainImpl::ActCommit_activated() {
 
-	ActCommit_setEnabled(false);
-	CommitImpl* commitViewer = new CommitImpl(git, &changesCommitted); // has WDestructiveClose
-	connect(this, SIGNAL(closeAllWindows()), commitViewer, SLOT(close()));
-	connect(commitViewer, SIGNAL(destroyed()), this, SLOT(slotCommitViewerClosed()));
-	commitViewer->show();
+	CommitImpl* c = new CommitImpl(git); // has WDestructiveClose
+
+	connect(this, SIGNAL(closeAllWindows()), c, SLOT(close()));
+	connect(c, SIGNAL(changesCommitted(bool)), this, SLOT(on_changesCommitted(bool)));
+	c->show();
+}
+
+void MainImpl::on_changesCommitted(bool ok) {
+
+	if (ok)
+		refreshRepo(false);
+	else
+		statusBar()->message("Failed to commit changes");
 }
 
 void MainImpl::ActCommit_setEnabled(bool b) {
@@ -1471,18 +1479,6 @@ void MainImpl::ActCommit_setEnabled(bool b) {
 	ActCommit->setEnabled(b);
 }
 
-void MainImpl::slotCommitViewerClosed() {
-
-	if (changesCommitted) {
-		changesCommitted = false;
-		refreshRepo(false);
-	} else {
-		QListViewItem* item = rv->tab()->listViewLog->findItem(ZERO_SHA, COMMIT_COL);
-		ActCommit_setEnabled(item != NULL && !git->isCommittingMerge());
-		statusBar()->message("Failed to commit changes");
-	}
-}
-
 void MainImpl::ActTag_activated() {
 
 	int adj = -1; // hack to correctly map col numbers in main view
@@ -1653,7 +1649,9 @@ void MainImpl::ActHelp_activated() {
 
 	// helpInfo is defined in help.h
 	HelpBase* helpDlg = new HelpBase(NULL, 0, Qt::WDestructiveClose);
+
 	connect(this, SIGNAL(closeAllWindows()), helpDlg, SLOT(close()));
+
 	helpDlg->textEditHelp->setText(QString::fromLatin1(helpInfo));
 	helpDlg->show();
 	helpDlg->raise();
@@ -1663,7 +1661,7 @@ void MainImpl::ActAbout_activated() {
 
 	static const char* aboutMsg =
 	"<center><p><b>QGit version " PACKAGE_VERSION "</b></p><br>"
-	"<p>Copyright  2005, 2006 Marco Costalba</p>"
+	"<p>Copyright (c) 2005, 2006 Marco Costalba</p>"
 	"<p>Use and redistribute under the "
 	"terms of the GNU General Public License</p></center>";
 	QMessageBox::about(this, "About QGit", QString::fromLatin1(aboutMsg));
@@ -1682,13 +1680,13 @@ void MainImpl::closeEvent(QCloseEvent* ce) {
 	// when childern are deleted, d'tor do not call close() anymore. So we miss
 	// lastWindowClosed() signal in this case.
 	emit closeAllWindows();
-
 	hide();
 
 	EM_RAISE(exExiting);
+
 	if (!git->stop(Git::optSaveCache)) {
-		// not all processes have been deleted, there is
-		// still some run() not returned somewhere, it is
+		// if not all processes have been deleted, there is
+		// still some run() call not returned somewhere, it is
 		// not safe to delete run() callers objects now
 		QTimer::singleShot(100, this, SLOT(ActClose_activated()));
 		return;
