@@ -11,48 +11,47 @@
 #include <qregexp.h>
 #include <qprocess.h>
 #include <qptrvector.h>
+#include <qtimer.h>
 #include "exceptionmanager.h"
 #include "common.h"
 #include "mainbase.h"
 
 class QListViewItem;
 class QListBoxItem;
-class Git;
-class Rev;
-class Domain;
-class CommitImpl;
-class QProcess;
 class QPopupMenu;
 class QCustomEvent;
 class QAccel;
 class QTextEdit;
+class Git;
+class Domain;
 class ListViewItem;
-class TreeView;
-class PatchView;
-class FileView;
 class RevsView;
-class TabFile;
+class FileView;
 class FileHistory;
 
 class MainImpl : public MainBase {
 Q_OBJECT
 public:
-	MainImpl(const QString& curDir = "", QWidget* parent = 0, const char* name = 0);
+	MainImpl(const QString& curDir = "", QWidget* parent = 0);
+	QString curWorkDir() const { return curDir; }
 
 signals:
 	void highlightPatch(const QString&, bool);
+	void updateRevDesc();
 	void closeAllWindows();
 	void closeAllTabs();
 	void tabClosed(int tabPos);
+	void repaintListViews(const QFont&);
+	void closeTabButtonEnabled(bool);
 
 public slots:
 	void tabWdg_currentChanged(QWidget*);
 	void on_newRevsAdded(const FileHistory*, const QValueVector<QString>&);
 
 protected:
-	void customEvent(QCustomEvent* e);
+	virtual void customEvent(QCustomEvent* e);
 
-private slots:
+protected slots:
 	void initWithEventLoopActive();
 	void refreshRepo(bool setCurRevAfterLoad = true);
 	void on_listViewLog_doubleClicked(QListViewItem*);
@@ -64,7 +63,7 @@ private slots:
 	void on_customAction_activated(int id);
 	void on_customAction_exited(const QString& name);
 	void on_goRef_activated(int id);
-	void slotCommitViewerClosed();
+	void on_changesCommitted(bool);
 	void lineEditSHA_returnPressed();
 	void lineEditFilter_returnPressed();
 	void pushButtonCloseTab_clicked();
@@ -107,15 +106,14 @@ private slots:
 private:
 	friend class FileView;
 	friend class RevsView;
-	friend class ListView;
 	friend class PatchView;
+	friend class setRepoDelayed;
 
-	bool eventFilter(QObject* obj, QEvent* ev);
+	virtual bool eventFilter(QObject* obj, QEvent* ev);
 	void updateGlobalActions(bool b);
 	void updateContextActions(SCRef newRevSha, SCRef newFileName, bool isDir, bool found);
-	void setupPixmaps(int h);
 	void setupAccelerator(QAccel* accel);
-	int currentTabType(QObject** t);
+	int currentTabType(Domain** t);
 	void filterList(bool isOn, bool onlyHighlight);
 	bool passFilter(ListViewItem* i, SCRef f, int cn, const QMap<QString,bool>& sm);
 	void setRepository(SCRef wd, bool r, bool ks, QStringList* fl = NULL);
@@ -139,12 +137,10 @@ private:
 	template<class X> X* firstTab(int startPos = -1);
 	void openFileTab(FileView* fv = NULL);
 
-	EM_DECLARE(exSetRepositoryCalled);
 	EM_DECLARE(exExiting);
 
 	Git* git;
 	RevsView* rv;
-	QAccel* accel;
 
 	// curDir is the repository working dir, could be different from qgit running
 	// directory QDir::current(). Note that qgit could be run from subdirectory
@@ -155,13 +151,9 @@ private:
 	QRegExp longLogRE;
 	QGuardedPtr<QPopupMenu> contextMenu;
 	QGuardedPtr<QPopupMenu> contextSubMenu;
-	QPtrVector<QPixmap> pixmaps;
 	QString startUpDir;
 	QString textToFind;
-	bool changesCommitted;
-	int ph; // pixmaps height
-	int pw; // pixmaps width
-	unsigned long secs;
+	QFont listViewFont;
 	bool setRepositoryBusy;
 	int recentRepoMenuPos;
 };
@@ -171,15 +163,15 @@ Q_OBJECT
 public:
 	ExternalDiffProc(const QStringList& args, QObject* p) : QProcess(args, p) {
 
-		connect(this, SIGNAL(processExited()), this, SLOT(exit()));
+		connect(this, SIGNAL(processExited()), this, SLOT(on_processExited()));
 	}
 	~ExternalDiffProc() {
 
 		tryTerminate();
 		removeFiles();
 	}
-public slots:
-	void exit() { deleteLater(); }
+private slots:
+	void on_processExited() { deleteLater(); }
 
 private:
 	void removeFiles() {
@@ -192,4 +184,25 @@ private:
 	}
 };
 
+class setRepoDelayed : public QObject {
+Q_OBJECT
+public:
+	setRepoDelayed(MainImpl* mi, SCRef nd, bool r, bool ks, QStringList* fl) :
+	QObject(mi), m(mi), newDir(nd), refresh(r), keepSelection(ks), filterList(fl) {
+
+		QTimer::singleShot(100, this, SLOT(on_timeout()));
+	}
+private slots:
+	void on_timeout() {
+
+		m->setRepository(newDir, refresh, keepSelection, filterList);
+		deleteLater();
+	 }
+private:
+	MainImpl* m;
+	const QString newDir;
+	bool refresh, keepSelection;
+	QStringList* filterList;
+};
+
 #endif
