@@ -20,6 +20,7 @@ public:
 	MyProcess(QObject *go, Git* g, const QString& wd, bool reportErrors);
 	bool runSync(SCRef runCmd, QString* runOutput, QObject* rcv, SCRef buf);
 	bool runAsync(SCRef rc, QObject* rcv, SCRef buf);
+	static void parseArgs(SCRef cmd, SList args);
 
 signals:
 	void procDataReady(const QString&);
@@ -37,8 +38,8 @@ private:
 	void setupSignals();
 	bool launchMe(SCRef runCmd, SCRef buf);
 	void sendErrorMsg(bool notStarted = false);
-	void restoreSpaces(QString& newCmd, SCRef sepChar);
-	const QStringList splitArgList(SCRef cmd);
+	static void restoreSpaces(QString& newCmd, SCRef sepChar);
+	static const QStringList splitArgList(SCRef cmd);
 
 	QObject* guiObject;
 	Git* git;
