@@ -20,7 +20,7 @@ class Domain;
 class RevDesc: public QTextBrowser {
 Q_OBJECT
 public:
-	RevDesc(QWidget* parent = 0, const char* name = 0);
+	RevDesc(QWidget* parent, const char* name);
 	void setDomain(Domain* dm) { d = dm; };
 
 protected:
