@@ -41,7 +41,7 @@ RevsView::RevsView(MainImpl* mi, Git* g) : Domain(mi, g) {
 	m()->tabWdg->addTab(revTab, "&Rev list");
 	tabPosition = m()->tabWdg->count() - 1;
 
-	listViewLog = new ListView(this, git, tab()->listViewLog, NULL);
+	listViewLog = new ListView(this, git, tab()->listViewLog, NULL, m()->listViewFont);
 	tab()->textBrowserDesc->setDomain(this);
 	listBoxFiles = new ListBoxFiles(this, git, tab()->listBoxFiles);
 	treeView = new TreeView(this, git, m()->treeView);
@@ -50,15 +50,19 @@ RevsView::RevsView(MainImpl* mi, Git* g) : Domain(mi, g) {
 	listViewLog, SLOT(on_newRevsAdded(const FileHistory*, const QValueVector<QString>&)));
 
 	connect(git, SIGNAL(loadCompleted(const FileHistory*, const QString&)),
-		this, SLOT(on_loadCompleted(const FileHistory*, const QString&)));
+	        this, SLOT(on_loadCompleted(const FileHistory*, const QString&)));
+
+	connect(m(), SIGNAL(repaintListViews(const QFont&)),
+	        listViewLog, SLOT(on_repaintListViews(const QFont&)));
+
+	connect(m(), SIGNAL(updateRevDesc()), this, SLOT(on_updateRevDesc()));
 
 	connect(listViewLog, SIGNAL(lanesContextMenuRequested(const QStringList&,
 	        const QStringList&)), this, SLOT(on_lanesContextMenuRequested
-	        (const QStringList&, const QStringList&)));
+	       (const QStringList&, const QStringList&)));
 
-	connect(listViewLog, SIGNAL(droppedRevisions(const QString&,
-	        const QStringList&, const QString&)), this, SLOT(on_droppedRevisions
-	        (const QString&, const QStringList&, const QString&)));
+	connect(listViewLog, SIGNAL(droppedRevisions(const QStringList&)),
+	        this, SLOT(on_droppedRevisions(const QStringList&)));
 
 	connect(listViewLog, SIGNAL(contextMenu(const QString&, int)),
 	        this, SLOT(on_contextMenu(const QString&, int)));
@@ -118,10 +122,12 @@ void RevsView::viewPatch(bool newTab) {
 
 		connect(linkedPatchView->tab()->listBoxFiles, SIGNAL(doubleClicked(QListBoxItem*)),
 			m(), SLOT(on_fileList_doubleClicked(QListBoxItem*)));
+
+		connect(m(), SIGNAL(updateRevDesc()), pv, SLOT(on_updateRevDesc()));
 	}
 	connect(m(), SIGNAL(closeAllTabs()), pv, SLOT(on_closeAllTabs()));
 	pv->st = st;
-	UPDATE_DM_MASTER(pv);
+	UPDATE_DM_MASTER(pv, false);
 }
 
 void RevsView::on_loadCompleted(const FileHistory* fh, const QString& stats) {
@@ -130,9 +136,10 @@ void RevsView::on_loadCompleted(const FileHistory* fh, const QString& stats) {
 		return;
 
 	if (st.sha().isEmpty()) { // point to first one in list
-		if (tab()->listViewLog->firstChild()) {
-			SCRef firstRev(tab()->listViewLog->firstChild()->text(QGit::COMMIT_COL));
-			st.setSha(firstRev);
+
+		QListViewItem* item = tab()->listViewLog->firstChild();
+		if (item) {
+			st.setSha(((ListViewItem*)item)->sha());
 			st.setSelectItem(true);
 		}
 	}
@@ -140,90 +147,71 @@ void RevsView::on_loadCompleted(const FileHistory* fh, const QString& stats) {
 	QApplication::postEvent(this, new MessageEvent(stats));
 }
 
-bool RevsView::doUpdate() {
+void RevsView::on_updateRevDesc() {
 
-	try {
-		EM_REGISTER(m()->exSetRepositoryCalled);
-		EM_REGISTER(m()->exExiting);
+	SCRef d(git->getDesc(st.sha(), m()->shortLogRE, m()->longLogRE));
+	tab()->textBrowserDesc->setText(d);
+	tab()->textBrowserDesc->setCursorPosition(0, 0);
+}
 
-		bool dataCleared = m()->lineEditSHA->text().isEmpty();
-		bool found = listViewLog->update();
+bool RevsView::doUpdate(bool force) {
 
-		if (!found) { // views are updated only if sha is found
+	force = force || m()->lineEditSHA->text().isEmpty();
 
-			const QString tmp("Sorry, revision " + st.sha() +
-			                  " has not been found in main view");
-			m()->statusBar()->message(tmp);
+	bool found = listViewLog->update();
 
-		} else {
-			bool shaChanged = (st.sha(true) != st.sha(false));
-			bool diffToChanged = (st.diffToSha(true) != st.diffToSha(false));
-			bool allChanged = (st.allMergeFiles(true) != st.allMergeFiles(false));
+	if (!found && !st.sha().isEmpty()) {
 
-			if (shaChanged || dataCleared) {
+		const QString tmp("Sorry, revision " + st.sha() +
+		                  " has not been found in main view");
+		m()->statusBar()->message(tmp);
 
-				updateLineEditSHA();
-				SCRef d(git->getDesc(st.sha(), m()->shortLogRE, m()->longLogRE));
-				tab()->textBrowserDesc->setText(d);
-				tab()->textBrowserDesc->setCursorPosition(0, 0);
+	} else { // sha could be NULL
 
-				m()->statusBar()->message(git->getRevInfo(st.sha(), false));
-			}
-			const RevFile* files = NULL;
+		if (st.isChanged(StateInfo::SHA) || force) {
 
-			if (shaChanged || diffToChanged || allChanged || dataCleared)
-				// could call processEvents()
-				files = git->getFiles(st.sha(), st.diffToSha(), st.allMergeFiles());
-
-			listBoxFiles->update(files);
-
-			if (m()->treeView->isVisible())
-				treeView->update(); // blocking call
-
-			if (st.selectItem()) {
-				bool isDir = treeView->isDir(st.fileName());
-				m()->updateContextActions(st.sha(), st.fileName(), isDir, found);
-			}
-			// at the end update diffs that is the slowest and must be
-			// run after update of file list for 'diff to sha' to work
-			if (linkedPatchView) {
-				linkedPatchView->st = st;
-				UPDATE_DM_MASTER(linkedPatchView); // async call
-			}
+			updateLineEditSHA();
+			on_updateRevDesc();
+			m()->statusBar()->message(git->getRevInfo(st.sha(), false));
 		}
-		if (!found && dataCleared && tab()->listViewLog->currentItem()) {
-			// we are in an inconsistent state: list view current item is
-			// not selected and secondary panes are empty.
-			// This could happen as example after removing a tree filter.
-			// At least populate secondary panes
-			SCRef firstRev(tab()->listViewLog->currentItem()->text(QGit::COMMIT_COL));
-			st.setSha(firstRev);
-			st.setSelectItem(false);
-			customEvent(new UpdateDomainEvent(false)); // will be queued immediately
+		const RevFile* files = NULL;
+		bool newFiles = false;
+
+		if (st.isChanged(StateInfo::ANY & ~StateInfo::FILE_NAME) || force) {
+
+			files = git->getFiles(st.sha(), st.diffToSha(), st.allMergeFiles());
+			newFiles = true;
 		}
-		EM_REMOVE(m()->exExiting);
-		EM_REMOVE(m()->exSetRepositoryCalled);
+		// call always to allow a simple refresh
+		listBoxFiles->update(files, newFiles);
 
-		return found;
+		// update the tree at startup or when releasing a no-match toolbar search
+		if (m()->treeView->isVisible() || st.sha(false).isEmpty())
+			treeView->update(); // blocking call
 
-	} catch(int i) {
-
-		EM_REMOVE(m()->exExiting);
-		EM_REMOVE(m()->exSetRepositoryCalled);
-
-		if (EM_MATCH(i, m()->exSetRepositoryCalled, "update views")) {
-			EM_CHECK_PENDING;
-			return false;
+		if (st.selectItem()) {
+			bool isDir = treeView->isDir(st.fileName());
+			m()->updateContextActions(st.sha(), st.fileName(), isDir, found);
 		}
-		if (EM_MATCH(i, m()->exExiting, "update main view")) {
-			EM_CHECK_PENDING;
-			return false;
+		// at the end update diffs that is the slowest and must be
+		// run after update of file list for 'diff to sha' to work
+		if (linkedPatchView) {
+			linkedPatchView->st = st;
+			UPDATE_DM_MASTER(linkedPatchView, force); // async call
 		}
-		const QString info("Exception \'" + EM_DESC(i) + "\' "
-		                   "not handled in main view update...re-throw");
-		dbp("%1", info);
-		throw i;
 	}
+	QListViewItem* item = tab()->listViewLog->currentItem();
+
+	if (item && item->isVisible() && !found && force) {
+		// we are in an inconsistent state: list view current item is
+		// not selected and secondary panes are empty.
+		// This could happen as example after removing a tree filter.
+		// At least populate secondary panes
+		st.setSha(((ListViewItem*)item)->sha());
+		st.setSelectItem(false);
+		customEvent(new UpdateDomainEvent(false)); // will be queued immediately
+	}
+	return (found || st.sha().isEmpty());
 }
 
 void RevsView::updateLineEditSHA(bool clear) {
@@ -252,39 +240,34 @@ void RevsView::on_lanesContextMenuRequested(SCList parents, SCList childs) {
 
 	QPopupMenu contextMenu;
 	uint i = 0;
-	QStringList::const_iterator it(childs.constBegin());
-	for ( ; it != childs.constEnd(); ++it, i++)
-		contextMenu.insertItem("Child: " + git->getShortLog(*it), i);
+	loopList(c, childs)
+		contextMenu.insertItem("Child: " + git->getShortLog(*c), i++);
 
-	for (it = parents.constBegin() ; it != parents.constEnd(); ++it, i++) {
+	loopList(p, parents) {
 
-		QString log(git->getShortLog(*it));
+		QString log(git->getShortLog(*p));
 		if (log.isEmpty())
-			log = *it;
+			log = *p;
 
-		contextMenu.insertItem("Parent: " + log, i);
+		contextMenu.insertItem("Parent: " + log, i++);
 	}
 	int id = contextMenu.exec(QCursor::pos()); // modal exec
 	if (id == -1)
 		return;
 
 	int cc = (int)childs.count();
-	SCRef target((id < cc) ? childs[id] : parents[id - cc]);
+	SCRef target(id < cc ? childs[id] : parents[id - cc]);
 	st.setSha(target);
 	UPDATE();
 }
 
-void RevsView::on_droppedRevisions(const QString& sha, const QStringList& remoteRevs,
-                                   const QString& remoteRepo) {
+void RevsView::on_droppedRevisions(SCList remoteRevs) {
+// remoteRevs is already sanity checked to contain some possible valid data
 
 	if (isDropping()) // avoid reentrancy
 		return;
 
-	QDir dr;
-	if (!dr.exists(remoteRepo))
-		return;
-
-	dr.setPath(m()->curDir + QGit::PATCHES_DIR);
+	QDir dr(m()->curDir + QGit::PATCHES_DIR);
 	if (dr.exists()) {
 		const QString tmp("Please remove stale import directory " + dr.absPath());
 		m()->statusBar()->message(tmp);
@@ -294,24 +277,29 @@ void RevsView::on_droppedRevisions(const QString& sha, const QStringList& remote
 	if (!m()->askApplyPatchParameters(&commit, &fold))
 		return;
 
-	// ok, let's go.
-	dr.setFilter(QDir::Files);
+	// ok, let's go
 	setDropping(true);
+	dr.setFilter(QDir::Files);
 	QApplication::setOverrideCursor(QCursor(Qt::WaitCursor));
 	m()->raise();
 	EM_PROCESS_EVENTS;
 
-	uint applied = 0;
-	for (uint i = 1; i <= remoteRevs.count(); i++) {
+	uint revNum = 0;
+	QStringList::const_iterator it(remoteRevs.constEnd());
+	do {
+		--it;
 
-		SCRef sha(remoteRevs[remoteRevs.count() - i].section('@', 0, 0));
-		const QStringList shaList(sha); // we create patches one by one
+		QString tmp("Importing revision %1 of %2");
+		m()->statusBar()->message(tmp.arg(++revNum).arg(remoteRevs.count()));
 
-		const QString tmp(QString("Importing revision %1 of %2")
-		                  .arg(i).arg(remoteRevs.count()));
-		m()->statusBar()->message(tmp);
+		SCRef sha((*it).section('@', 0, 0));
+		SCRef remoteRepo((*it).section('@', 1));
 
-		if (!git->formatPatch(shaList, dr.absPath(), remoteRepo))
+		if (!dr.exists(remoteRepo))
+			break;
+
+		// we create patches one by one
+		if (!git->formatPatch(QStringList(sha), dr.absPath(), remoteRepo))
 			break;
 
 		dr.refresh();
@@ -320,22 +308,23 @@ void RevsView::on_droppedRevisions(const QString& sha, const QStringList& remote
 			       "in %s", dr.count(), QGit::PATCHES_DIR.latin1());
 			break;
 		}
-		bool ok = git->applyPatchFile(dr.absFilePath(dr[0]), commit, fold, !Git::optSign);
-		dr.remove(dr[0]);
-		if (!ok)
+		SCRef fn(dr.absFilePath(dr[0]));
+		if (!git->applyPatchFile(fn, commit, fold, Git::optDragDrop))
 			break;
 
-		applied = i;
-	}
-	if (applied != remoteRevs.count())
-		m()->statusBar()->message("Failed to import revision " + sha);
-	else
+		dr.remove(fn);
+
+	} while (it != remoteRevs.constBegin());
+
+	if (it == remoteRevs.constBegin())
 		m()->statusBar()->clear();
+	else
+		m()->statusBar()->message("Failed to import revision " + QString::number(revNum--));
 
-	if (!commit && (applied > 0))
-		git->resetCommits(applied);
+	if (!commit && (revNum > 0))
+		git->resetCommits(revNum);
 
-	dr.rmdir(dr.absPath());
+	dr.rmdir(dr.absPath()); // 'dr' must be already empty
 	QApplication::restoreOverrideCursor();
 	setDropping(false);
 	m()->refreshRepo();
